#!/bin/sh 
shopt -s nocasematch
date=$(echo `date | cut -d' ' -f2,3,4 | sed "s/  /./g" | sed "s/ /./g" | sed "s/:/./g"`)
##colors
lb='\033[1;34m'
lg='\033[1;32m'
red='\033[0;31m'
NC='\033[0m'
yes='\xE2\x9C\x93'
no='\xE2\x9C\x97'

#functions
function checkString () {
   success=
   choice=$1
   options=$2
   if [[ ! $(echo "$options" | grep -i -w "$choice") ]]; then
      echo -e "${red}You did not type a valid option. Try again.${NC}"
   else
      success=true
   fi
}

function checkTools () {
   tools=$1
   all_tools=$2
   for tool in $tools; do
      if [ ! "$(echo $all_tools | grep -w "$tool")" ]; then
         echo -e "$tool ${red}is not a valid option.${NC}"
         success=
         break
      else
         success=true
      fi
   done
}

function dirExists () {
   success=
   if [[ ! -e $1 ]]; then
      echo -e "${red}Specified path doesn't exist.${NC}"
   else
      success=true
   fi
}

function checkParent () {
   success=
   if [ "$(echo $1 | head -c 1)" = "/" ]; then
		if [ ! -d $(dirname $1) ]; then
           echo -e "${red}Parent directory doesn't exist. Try again.${NC}"
        else
           success=true
		fi
	else 
		echo -e "${red}You did not enter a valid path.${NC}"
   fi
}

function checkUser () {
   success=
   if [ $(echo $1 | wc -w) -gt 1 ]; then
      for x in $1; do
         for line in $(cat /etc/passwd); do
            if [ $(echo $line | cut -d':' -f1 | grep "$x") ]; then
               user=$(echo $line | cut -d':' -f1 | grep -w "$x")
               break
            else
               user=
            fi       
         done
         if [ ! $user -o $user = "root" ]; then
            echo -e "$x ${red}is an invalid user. Try again.${NC}"
            success=
            break
         else
            success=true
         fi
      done
   elif [ $(echo $1 | wc -w) == 0 ]; then 
      success=true #if you leave user blank, it will set variable to empty and move on
   else
      for line in $(cat /etc/passwd); do
         if [ $(echo $line | cut -d':' -f1 | grep "$1") ]; then
            user=$(echo $line | cut -d':' -f1 | grep -w "$1")
            break
         else 
            user=
         fi
      done
   
      if [ ! $user -o $user = "root" ]; then
         echo -e "${red}Invalid user. Try again.${NC}"
      else
         success=true
      fi
   fi
}

remove_oldest() {
   oldf=
   for f in ./install_files/preferences*; do
      # not a file, ignore
      [[ ! -f "$f" ]] && continue
      # find oldest entry
      [[ -z "$oldf" ]] && oldf="$f" || [[ "$f" -ot "$oldf" ]] && oldf="$f"
   done
   echo "removing old file $oldf"
   rm -f $oldf
}

##remove oldest preferences file if there are more than 5 to prevent too many from building up
if [ $(ls -1 ./install_files/preferences* | wc -w) -gt 3 ]; then
   remove_oldest
fi

echo -e "${lb}Preferences file creator: Answer the following questions to create a preferences file. \nThe automatic installer will be launched afterwards.${NC}"
echo -e "${lb}======================================================================================${NC}"

while
   echo -e "Enter the path of your Shared File System ${lg}(ex:/usr/local)${NC}: \c"
   read sfs
   dirExists $sfs
   [ "$success" != "true" ] 
do :; done

echo -e "Install LONI Pipeline's server-side application ${lg}(y/n)${NC}? \c"
read installPipeline
if [[ "$installPipeline" = "Y" ]] || [[ "$installPipeline" = "yes" ]]; then
   #echo -e "${yes}"
   installPipeline=true
else 
   #echo -e "${no}"
   installPipeline=false
fi

if [[ "$installPipeline" = "true" ]]; then
   echo -e "${lb}===================Pipeline Server Installation==================${NC}"
   
   while   
      echo -e "Enter the directory where you want the Pipeline server installed ${lg}(ex:/usr/pipeline)${NC}: \c"
      read pipelineDir
      checkParent $pipelineDir
      [ "$success" != "true" ]
   do :; done

   while
      echo -e "Enter the directory where you want the Pipeline Database to be installed ${lg}(ex:/usr/pipeline/db)${NC}: \c"
      read dbDir
      checkParent $dbDir
      [ "$success" != "true" ]
   do :; done

   echo -e "Enter the network port you want the Pipeline server to use ${lg}(default=8001)${NC}: \c"
   read pipelinePort
   if [ -z $pipelinePort ]; then
      pipelinePort=8001
	  echo "Set to 8001."
   fi
   
   while   
      echo -e "Enter the user who will run the pipeline server process ${lg}(user must exist and not be root)${NC}: \c"
      read pipelineUsername
      checkUser "$pipelineUsername"
      [ "$success" != "true" ]
   do :; done

   echo -e "Enable grid submission ${lg}(y/n)${NC}? \c"
   read gridSub
   if [[ "$gridSub" = "Y" ]]; then
      #echo -e "${yes}"
      gridSub=true
      while    
         echo -e "Choose the grid plugin to use. Enter ${lg}JGDI${NC} or ${lg}DRMAA${NC}. Note: JGDI is more stable and thus recommended: \c"
         read gridPlugin
         gridPlugin=$(echo $gridPlugin | awk '{print toupper($0)}')
         checkString $gridPlugin "JGDI DRMAA"
         [ "$success" != "true" ]
      do :; done
      
      which qconf >> /dev/null 2>&1
      if [ $(echo $?) ==  0 ]; then
         queuelist=$(echo $(qconf -sql))
         while
            echo -e "Select an existing queue to where Pipeline will submit it's jobs: ${lg}$queuelist${NC}  >\c"
            read gridQueue
            checkString $gridQueue "$(echo $queuelist)"
            [ "$success" != "true" ]
         do :; done
      fi
   else
	  #echo -e "${no}"
      gridSub=false
   fi

   echo -e "Use privilege escalation? This will cause Pipeline to sudo as users when submitting jobs ${lg}(y/n)${NC}: \c"
   read privEsc   
   if [[ "$privEsc" = "y" ]] || [[ "$privEsc" = "yes" ]]; then
		#echo -e "${yes}"
		privEsc=true
	else
		privEsc=false
		#echo -e "${no}"
	fi
 
   while
      echo -e "Which authentication method would you like to use for users who connect to the Pipeline server? Please choose from the following: ${lg}SSH${NC}, ${lg}NIS${NC}, or ${lg}none${NC}  >\c"
      read auth
      checkString "$auth" "SSH NIS none"
      if [[ "$auth" = "none" ]]; then
         auth=NoAuth
	  else 
         auth=$(echo $auth | awk '{print toupper($0)}')
	  fi
      [ "$success" != "true" ]
   do :; done
        
   echo -e "Modify the sudoers file to add superusers who cannot be sudoed as by the Pipeline User ${lg}(y/n)${NC}? \c"
   read modifySudoers
   if [[ "$modifySudoers" = "Y" ]] || [[ "$modifySudoers" = "yes" ]]; then
      modifySudoers=true
	  #echo -e "${yes}"
      while
         echo -e "Which users would you like to add as superusers? Leave blank if you only want root in this list, otherwise ${lg}enter 1 or more users${NC}, space-separated: \c".
         read superUsers
         checkUser "$superUsers"
         [ "$success" != "true" ]
      do 
         if [ ! -z "$superUsers" ]; then
             superUsers=root,$superUsers
         else 
             superUsers=root
         fi
      done
   else
      modifySudoers=false
	  #echo -e "${no}"
   fi
	
   re='^[0-9]+$'
   echo -e "Choose the amount of memory allocation (in mb) for the Pipeline server. Leave this blank to have this determined automatically: \c"
   read memAlloc
   if ! [[ $memAlloc =~ $re ]] ; then
		echo -e "${red}No number entered. Going with default.${NC}" 
   fi

   echo -e "Start the Pipeline server automatically each time you start/reboot the machine ${lg}(y/n)${NC}? \c"
   read startPipeline
   if [ "$startPipeline" = "y" ] || [ "$startPipeline" = "yes" ]; then
      startPipeline=true 
	  #echo -e "${yes}"
   else 
      startPipeline=false
	  #echo -e "${no}"
   fi  

fi   
   
echo -e "Install the Grid Engine (job submission application for a cluser with multiple hosts)? This is not required for the pipeline server to function ${lg}(y/n)${NC}: \c"
read installSGE
if [[ "$installSGE" = "Y" ]] || [[ "$installSGE" = "yes" ]]; then
   installSGE=true
   #echo -e "${yes}"
else
   installSGE=false
   #echo -e "${no}"
fi
 
if [[ "$installSGE" = "true" ]]; then
   echo -e "${lb}=============================SGE Installation===================================${NC}"
   
   while
      echo -e "Enter the directory where you want SGE to be installed ${lg}(ex:/usr/local/sge)${NC}: \c"
      read sgeDir
      checkParent $sgeDir
      [ "$success" != "true" ]
   do :; done
   
   echo -e "Enter your ${lg}SGE cluster name${NC}. Hit enter to keep the default name, cluster: \c"
   read clusterName
   if [[ -z $clusterName ]]; then 
      clusterName=cluster
   fi
  
   echo -e "Enter the ${lg}SGE spool directory${NC} location. This must be a shared directory accessible by all execution hosts ${lg}(ex:/usr/local/sge/spool)${NC}: \c"
   read spoolDir
   #check

   echo -e "If you would like to set a custom SGE admin user, please ${lg}enter their username${NC}. Otherwise, hit enter: \c"
   read adminUser
   if [[ -z $adminUser ]]; then
      adminUser=root #should admin user always include root?
   fi
   
   echo -e "Enter execution host(s): \c" 
   read execHosts
   echo -e "Enter submit host(s): \c"
   read submitHosts
   echo -e "Enter admin host(s): \c"
   read adminHosts

   #queue configure
fi

#tools installation
toolsPath=null
while [ "$toolsPath" = "null" ]; do
	echo -e "If you want to install external neuro-imaging and/or bioinformatics tools, ${lg}enter an installation directory path${NC}, otherwise hit enter: \c"
	read toolsPath
	if [[ "$toolsPath" = "yes" ]] || [[ "$toolsPath" = "y" ]]; then
		toolsPath=null
	fi
done

if [[ ! -z "$toolsPath" ]] && [[ "$toolsPath" != "n" ]] && [[ "$toolsPath" != "no" ]]; then
   installTools=true
   
   while 
      echo -e "Which type of tools do you want to install, Neuro-imaging(i.e. AFNI,AIR,FSL), Bioinformaticc(i.e. MAQ,SamTools,Bowtie), or both? Please enter ${lg}NI${NC}, ${lg}BI${NC}, or ${lg}both${NC}: \c"
      read toolSelect
      checkString $toolSelect "NI BI both"
      [ "$success" != "true" ]
   do :; done

   bitools="emboss picard msa batwing bayesass formatomatic genepop migrate gwass mrfast bowtie samtools plink maq miblast all"
   if [[ "$toolSelect" = "NI" ]] || [[ "$toolSelect" = "both" ]]; then
      NITools=true

      while 
         echo -e "In the next step, you will select which NI tools to install. First indicate if you want to install the tool binaries, server library pipe files, or both by typing ${lg}exec${NC}, ${lg}serverlib${NC}, or ${lg}both${NC}: \c"
         read NIinstallType
         checkString $NIinstallType "exec serverlib both"
         [ "$success" != "true" ]
      do :; done

      niexecs="afni air loni minc itk gamma spm all"
      niservlibs="afni air fsl freesurfer loni minc itk dtk gamma spm all"

      if [[ "$NIinstallType" = "exec" ]]; then
         installniexecs=true
         while
            echo -e "From the following, indicate which NI tools you want to install by typing them as shown into the prompt: ${lg}$niexecs${NC}  >\c"
            read toolList
            checkTools "$toolList" "$niexecs"
            [ "$success" != "true" ]
         do :; done
            
      elif [[ "$NIinstallType" = "serverlib" ]]; then 
         installniservlib=true
         while 
            echo -e "From the following, indicate which NI tools you want to install by typing them as shown into the prompt: ${lg}$niservlibs${NC}  >\c"
            read toolList
            checkTools "$toolList" "$niservlibs"
            [ "$success" != "true" ]
         do :; done
      elif [[ "$NIinstallType" = "both" ]]; then
         installniexecs=true
         installniservlib=true
         while
            echo -e "From the following, indicate which NI tools you want to install by typing them as shown into the prompt: ${lg}$niservlibs${NC}  >\c"
            read toolList
			echo -e "${red}Note: Only library files for DTK, FSL, and FreeSurfer will be installed. To install their executables, you can use the launcher GUI, the -auto option, or install them manually.${NC}"
            checkTools "$toolList" "$niservlibs"
            [ "$success" != "true" ]
         do :; done
      else 
         echo -e "${red}You didn't type a valid option. ${red}"
         exit 
      fi

   #install$tools will look like installair, installafni, etc
      if [[ "$toolList" = "all" ]]; then
         for tool in $niservlibs; do
            eval install$tool=true
         done
      else 
         for tool in $niservlibs; do 
            if [[ $(echo $toolList | grep $tool) ]]; then
               eval install$tool=true
            else 
               eval install$tool=false
            fi
         done
      fi
   else 
      NITools=false
      installniexecs=false
      installniservlib=false
   fi
   if [[ "$toolSelect" = "BI" ]] || [[ "$toolSelect" = "both" ]]; then
      BITools=true
      while 
         echo -e "In the next step, you will select which BI tools to install. Please first indicate if you want to install the tool binaries, server library pipe files, or both by typing ${lg}exec${NC}, ${lg}serverlib${NC}, or ${lg}both${NC}: \c"      
         read BIinstallType
         checkString "$BIinstallType" "exec serverlib both"
         [ "$success" != "true" ]
      do :; done 
      if [[ "$BIinstallType" = "exec" ]]; then
         installbiexecs=true
      elif [[ "$BIinstallType" = "serverlib" ]]; then
         installbiservlib=true
      elif [[ "$BIinstallType" = "both" ]]; then
         installbiexecs=true
         installbiservlib=true
      else
         echo -e "${red}You didn't type a valid option. Exiting installer.${NC}"
         exit
      fi
    
     while
        echo -e "From the following list, please indicate which BI tools you want to install by typing them as shown into the prompt: ${lg}$bitools${NC}  >\c"
        read BItoolList
        checkTools "$BItoolList" "$bitools"
        [ "$success" != "true" ]
     do :; done

      if [[ "$BItoolList" = "all" ]]; then
         for tool in $bitools; do
            eval install$tool=true
         done
      else 
         for tool in $bitools; do 
            if [[ $(echo $BItoolList | grep $tool) ]]; then
               eval install$tool=true
            else 
               eval install$tool=false
            fi
         done
      fi

   else
      BITools=false
      installbiexecs=false
      installbiservlib=false
   fi
else
   toolsPath=
   installTools=false
fi
#echo "installair=$installair"
#echo "installafni=$installafni"
#echo "installfsl=$installfsl"
#echo "installfreesurfer=$installfreesurfer"
#echo "installloni=$installloni"
#echo "installminc=$installminc"
#echo "installitk=$installitk"
#echo "installdtk=$installdtk"
#echo "installgamma=$installgamma"
#echo "installspm=$installspm"
#echo "installemboss=$installemboss"
#echo "installpicard=$installpicard"

echo -en "${lb}Writing preferences file...${NC}"

echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>
<DistributedPipelineServerInstaller>
<SharedFileSystemPath>$sfs</SharedFileSystemPath>
<PipelineServer enabled=\"$installPipeline\">
    <InstallLocation>$pipelineDir</InstallLocation>
    <Port>$pipelinePort</Port>
    <Username>$pipelineUsername</Username>
    <GridSubmission enabled=\"$gridSub\">
        <GridPlugin>$gridPlugin</GridPlugin>
        <GridSubmissionQueue>$gridQueue</GridSubmissionQueue>
    </GridSubmission>
    <UsePrivilegeEscalation>$privEsc</UsePrivilegeEscalation>
    <DBInstallLocation>$dbDir</DBInstallLocation>
    <StartPipelineOnSystemStartup>$startPipeline</StartPipelineOnSystemStartup>
    <AuthenticationModule>$auth</AuthenticationModule>
    <ModifySudoers enabled=\"$modifySudoers\">
        <SuperUsers>$superUsers</SuperUsers>
    </ModifySudoers>
    <MemoryAllocation>$memAlloc</MemoryAllocation>
</PipelineServer>
<PreferencesPath>${pipelineDir}/preferences.xml</PreferencesPath>
<JDKLocation></JDKLocation>
<SGE enabled=\"$installSGE\">
    <SGERoot>$sgeDir</SGERoot>
    <SGECluster>$cluserName</SGECluster>
    <SubmitHosts>
        <Host>$submitHosts</Host>
    </SubmitHosts>
    <ExecHosts>
        <Host>$execHosts</Host>
    </ExecHosts>
    <AdminHosts>
        <Host>$adminHosts</Host>
    </AdminHosts>
    <AdminUsername>$adminUser</AdminUsername>
    <SpoolDir>$spoolDir</SpoolDir>
    <Queue configure=\"true\">
        <Name>$gridQueue</Name>
        <Hosts>
           <Host></Host>
        </Hosts>
        <Slots>
           <Host></Host>
        </Slots>
    </Queue>
</SGE>
<Tools enabled=\"$installTools\" path=\"$toolsPath\">
    <NITools enabled=\"$NITools\" executables=\"$installniexecs\" serverlib=\"$installniservlib\">
        <AFNI enabled=\"$installafni\" />
        <AIR enabled=\"$installair\" />
        <FSL enabled=\"$installfsl\">
           <ArchivePath></ArchivePath> 
        </FSL>
        <FreeSurfer enabled=\"$installfreesurfer\">
           <ArchivePath></ArchivePath>
        </FreeSurfer>
        <LONI enabled=\"$installloni\" />
        <MINC enabled=\"$installminc\" />
        <ITK enabled=\"$installitk\" />
        <DTK enabled=\"$installdtk\">
           <ArchivePath></ArchivePath>
        </DTK>
        <GAMMA enabled=\"$installgamma\" />
 	<SPM enabled=\"$installspm\" />
    </NITools>
    <BITools enabled=\"$BITools\" executables=\"$installbiexecs\" serverlib=\"$installbiservlib\">
        <EMBOSS enabled=\"$installemboss\" />
        <Picard enabled=\"$installpicard\" />
        <MSA enabled=\"$installmsa\" />
        <BATWING enabled=\"$installbatwing\" />
        <BayesAss enabled=\"$installbayesass\" />
        <Formatomatic enabled=\"$installformatomatic\" />
        <GENEPOP enabled=\"$installgenepop\" />
        <Migrate enabled=\"$installmigrate\" />
        <GWASS enabled=\"$installgwass\" />
        <MrFAST enabled=\"$installmrfast\" />
        <Bowtie enabled=\"$installbowtie\" />
        <SamTools enabled=\"$installsamtools\" />
        <PLINK enabled=\"$installplink\" />
        <MAQ enabled=\"$installmaq\" />
        <miBLAST enabled=\"$installmiblast\" />
    </BITools>
</Tools>
</DistributedPipelineServerInstaller>" > ./install_files/preferences_${date}.xml

##remove oldest preferences file if there are more than 5 to prevent too many from building up
if [ $(ls -1 ./install_files/preferences* | wc -w) -gt 5 ]; then
   remove_oldest
fi

echo -e "${lb}done! Launching installer...${NC}"
sleep 1s
../launchInstaller.sh -auto ./install_files/preferences_${date}.xml
