#!/bin/sh

echo "Installing $MANUALTOOL_NAME"

if [ "$INSTALLER_MODE_MANUAL_TOOL" = "true" ]
then
    rm -Rf ${TOOLS_PATH}/${MANUALTOOL_NAME}-${MANUALTOOL_VERSION} || exit 1
    if [ "$INSTALLER_MODE_MANUAL_CANCEL" = "true" ]
    then
		echo "Installation cancelled."	
		echo "nitools+=9"
		if [ "$MANUALTOOL_NAME" = "FSL" ] || [ "$MANUALTOOL_NAME" = "FreeSurfer" ]
		then
			echo "nitools++"
		fi
    elif [ "$MANUALTOOL_NAME" = "FSL" ] || [ "$MANUALTOOL_NAME" = "FreeSurfer" ] || [ "$MANUALTOOL_NAME" = "DTK" ]
    then
        # default value of $MANUALTOOL_ARCHIVE_PATH is arbitrarily set to $MANUALTOOL_NAME in InstallPanel.java
        if [ "$MANUALTOOL_NAME" != "$MANUALTOOL_ARCHIVE_PATH" ] 
        then
            cd $TOOLS_PATH
            echo "nitools-->Installing: "
            echo "nitools->${MANUALTOOL_NAME} v.${MANUALTOOL_VERSION}"
            tar xmvzf $MANUALTOOL_ARCHIVE_PATH -C $TOOLS_PATH || { echo "Error unzipping $MANUALTOOL_ARCHIVE_PATH"; exit 1; } 
                
			echo "nitools+=5"
			echo "Installing Tool:" $MANUALTOOL_NAME
               
            toolname_lowercase=`echo $MANUALTOOL_NAME | awk '{print tolower($0)}'`
            download_name=${toolname_lowercase}

			if [ ! -d ${TOOLS_PATH}/${toolname_lowercase} ]; then
                echo "${TOOLS_PATH}/${toolname_lowercase} does not exist; exiting" && exit 1
            fi

            chown -R $PL_USER $TOOLS_PATH/$toolname_lowercase || exit 1
            sudo -u $PL_USER mv $TOOLS_PATH/$toolname_lowercase $TOOLS_PATH/${MANUALTOOL_NAME}-${MANUALTOOL_VERSION} || exit 1
            sudo -u $PL_USER ln -s $TOOLS_PATH/${MANUALTOOL_NAME}-${MANUALTOOL_VERSION} $TOOLS_PATH/${MANUALTOOL_NAME} || exit 1
            echo "nitools+=2"
 
			# download and install glue
            cd $TOOLS_PATH/${MANUALTOOL_NAME}-${MANUALTOOL_VERSION}
            wget -c --progress=dot $server/${download_name}.tar.gz
            if [ $? -ne 0 ]; then
                echo "nitools!Failed to download ${download_name} binaries"
                exit 1
            fi
                 
            sudo -u $PL_USER tar xmvzf ${download_name}.tar.gz
            sudo -u $PL_USER rm ${download_name}.tar.gz
            echo "nitools+=2"

            if [ ! -z "$MANUALTOOL_LICENSE_PATH" ]
			then
                if [ "$MANUALTOOL_LICENSE_PATH" = ".license" ]; then 
					echo "No license provided."
				else
					cp $MANUALTOOL_LICENSE_PATH ${TOOLS_PATH}/${MANUALTOOL_NAME}-${MANUALTOOL_VERSION} || exit 1
                fi
			fi
				
			## we need extra scripts for freesurfer
			if [ "$MANUALTOOL_NAME" = "FreeSurfer" ] || [ "$MANUALTOOL_NAME" = "FSL" ]; then
				wget -c --progress=dot $server/${download_name}-scripts.tar.gz
				if [ $? -ne 0 ]; then
                    echo "nitools!Failed to download ${download_name} binaries"
					exit 1
                fi
				sudo -u $PL_USER tar xmvzf ${download_name}-scripts.tar.gz
				sudo -u $PL_USER rm ${download_name}-scripts.tar.gz
				echo "nitools++"
			fi
        fi

    ### SPM needs a special case
    elif [ "$MANUALTOOL_NAME" = "SPM8" ] && [ "$MCR_INSTALL" = "false" ]; then
    	cd $TOOLS_PATH
		### Install just SPM, not MCR
		echo "nitools-->Downloading: "; echo "nitools->SPM v.${MANUALTOOL_VERSION}"
		echo "nitools++"
		wget -c --progress=dot $server/spm8-${MANUALTOOL_VERSION}.tar.gz

		if [ $? -ne 0 ]; then echo "nitools!Failed to download SPM binaries"; exit 1; fi

		echo "nitools+=3"
		echo "nitools-->Installing: "; echo "nitools->SPM v.${MANUALTOOL_VERSION}"

		echo "nitools++"
		sudo -u $PL_USER tar xmvzf spm8-${MANUALTOOL_VERSION}.tar.gz || exit 1
		echo "nitools+=3"
		rm -f spm8-${MANUALTOOL_VERSION}.tar.gz || exit 1
		echo "nitools++"
		
    elif [ "$MANUALTOOL_NAME" = "SPM8" ] && [ "$MCR_INSTALL" != "false" ]; then
		cd $TOOLS_PATH
		echo "nitools-->Downloading: "; echo "nitools->SPM v.${MANUALTOOL_VERSION}"
		echo "nitools++"
		wget -c --progress=dot $server/spm8-${MANUALTOOL_VERSION}.tar.gz

		if [ $? -ne 0 ]; then echo "nitools!Failed to download SPM binaries"; exit 1; fi

		echo "nitools+=2"
		echo "nitools-->Installing: "; echo "nitools->SPM v.${MANUALTOOL_VERSION}"         
		echo "nitools++"
              
		sudo -u $PL_USER tar xmvzf spm8-${MANUALTOOL_VERSION}.tar.gz || exit 1
		echo "nitools+=2"
		rm -f spm8-${MANUALTOOL_VERSION}.tar.gz || exit 1
		echo "nitools++"

		##install the MCR 
		cd $TOOLS_PATH
		if [ ! -d MCR ]; then sudo -u $PL_USER mkdir MCR || exit 1; fi
		cd MCR || exit 1
                cp ${MCR_ARCHIVE_LOCATION} ./ || exit 1
		sudo -u $PL_USER unzip -o $(basename ${MCR_ARCHIVE_LOCATION}) || exit 1
		echo "nitools+=2"
		sudo -u $PL_USER ./install       
    fi
	if [ "$INSTALL_MANUALTOOL_SERVERLIB" = "true" ]
	then
		export CURRENT_TOOL=${MANUALTOOL_NAME}
		cd -
		. install_files/installWorkflows.sh
        if [ ! -z "${MANUALTOOL_VERSION}" ]
        then
			sudo -u $PL_USER sed -i "s/${MANUALTOOL_NAME}VERSION/${MANUALTOOL_VERSION}/g" ${PL_SERVERLIB}/*/*/*pipe
        fi
	fi
fi

