#!/bin/sh


echo "Your tools directory is $TOOLS_PATH"

if [ -z "$INSTALL_NI_TOOLS" ]
then
    echo "Please do not run this script manually. Run launchInstaller.sh instead."
    exit 1
fi

if [ "$INSTALLER_MODE_MANUAL_TOOL" = "true" ]
then
    rm -Rf ${TOOLS_PATH}/${MANUALTOOL_NAME}-${MANUALTOOL_VERSION} || exit 1
    if [ "$INSTALLER_MODE_MANUAL_CANCEL" = "true" ]
    then
		echo "nitools+=9"
		if [ "$MANUALTOOL_NAME" = "FSL" ] || [ "$MANUALTOOL_NAME" = "FreeSurfer" ]
		then
			echo "nitools++"
		fi
    elif [ "$MANUALTOOL_NAME" = "FSL" ] || [ "$MANUALTOOL_NAME" = "FreeSurfer" ] || [ "$MANUALTOOL_NAME" = "DTK" ]
    then
        # default value of $MANUALTOOL_ARCHIVE_PATH is arbitrarily set to $MANUALTOOL_NAME in InstallPanel.java
        if [ "$MANUALTOOL_NAME" != "$MANUALTOOL_ARCHIVE_PATH" ] 
        then
            cd $TOOLS_PATH
            echo "nitools-->Installing: "
            echo "nitools->${MANUALTOOL_NAME} v.${MANUALTOOL_VERSION}"
            tar xmvzf $MANUALTOOL_ARCHIVE_PATH -C $TOOLS_PATH || { echo "Error unzipping $MANUALTOOL_ARCHIVE_PATH"; exit 1; } 
                
			echo "nitools+=5"
			echo "Installing Tool:" $MANUALTOOL_NAME
               
            toolname_lowercase=`echo $MANUALTOOL_NAME | awk '{print tolower($0)}'`
            download_name=${toolname_lowercase}

			if [ ! -d ${TOOLS_PATH}/${toolname_lowercase} ]; then
                echo "${TOOLS_PATH}/${toolname_lowercase} does not exist; exiting" && exit 1
            fi

            chown -R $PL_USER $TOOLS_PATH/$toolname_lowercase || exit 1
            sudo -u $PL_USER mv $TOOLS_PATH/$toolname_lowercase $TOOLS_PATH/${MANUALTOOL_NAME}-${MANUALTOOL_VERSION} || exit 1
            sudo -u $PL_USER ln -s $TOOLS_PATH/${MANUALTOOL_NAME}-${MANUALTOOL_VERSION} $TOOLS_PATH/${MANUALTOOL_NAME} || exit 1
            echo "nitools+=2"
 
			# download and install glue
            cd $TOOLS_PATH/${MANUALTOOL_NAME}-${MANUALTOOL_VERSION}
            wget -c --progress=dot $server/${download_name}.tar.gz
            if [ $? -ne 0 ]; then
                echo "nitools!Failed to download ${download_name} binaries"
                exit 1
            fi
                 
            sudo -u $PL_USER tar xmvzf ${download_name}.tar.gz
            sudo -u $PL_USER rm ${download_name}.tar.gz
            echo "nitools+=2"

            if [ ! -z "$MANUALTOOL_LICENSE_PATH" ]
			then
                if [ "$MANUALTOOL_LICENSE_PATH" = ".license" ]; then 
					echo "No license provided."
				else
					cp $MANUALTOOL_LICENSE_PATH ${TOOLS_PATH}/${MANUALTOOL_NAME}-${MANUALTOOL_VERSION} || exit 1
                fi
			fi
				
			## we need extra scripts for freesurfer
			if [ "$MANUALTOOL_NAME" = "FreeSurfer" ] || [ "$MANUALTOOL_NAME" = "FSL" ]; then
				wget -c --progress=dot $server/${download_name}-scripts.tar.gz
				if [ $? -ne 0 ]; then
                    echo "nitools!Failed to download ${download_name} binaries"
					exit 1
                fi
				sudo -u $PL_USER tar xmvzf ${download_name}-scripts.tar.gz
				sudo -u $PL_USER rm ${download_name}-scripts.tar.gz
				echo "nitools++"
			fi
        fi

    ### SPM needs a special case
    elif [ "$MANUALTOOL_NAME" = "SPM8" ] && [ "$MCR_INSTALL" = "false" ]; then
    	cd $TOOLS_PATH
		### Install just SPM, not MCR
		echo "nitools-->Downloading: "; echo "nitools->SPM v.${MANUALTOOL_VERSION}"
		echo "nitools++"
		wget -c --progress=dot $server/spm8-${MANUALTOOL_VERSION}.tar.gz

		if [ $? -ne 0 ]; then echo "nitools!Failed to download SPM binaries"; exit 1; fi

		echo "nitools+=3"
		echo "nitools-->Installing: "; echo "nitools->SPM v.${MANUALTOOL_VERSION}"

		echo "nitools++"
		sudo -u $PL_USER tar xmvzf spm8-${MANUALTOOL_VERSION}.tar.gz || exit 1
		echo "nitools+=3"
		rm -f spm8-${MANUALTOOL_VERSION}.tar.gz || exit 1
		echo "nitools++"
		
    elif [ "$MANUALTOOL_NAME" = "SPM8" ] && [ "$MCR_INSTALL" != "false" ]; then
		cd $TOOLS_PATH
		echo "nitools-->Downloading: "; echo "nitools->SPM v.${MANUALTOOL_VERSION}"
		echo "nitools++"
		wget -c --progress=dot $server/spm8-${MANUALTOOL_VERSION}.tar.gz

		if [ $? -ne 0 ]; then echo "nitools!Failed to download SPM binaries"; exit 1; fi

		echo "nitools+=2"
		echo "nitools-->Installing: "; echo "nitools->SPM v.${MANUALTOOL_VERSION}"         
		echo "nitools++"
              
		sudo -u $PL_USER tar xmvzf spm8-${MANUALTOOL_VERSION}.tar.gz || exit 1
		echo "nitools+=2"
		rm -f spm8-${MANUALTOOL_VERSION}.tar.gz || exit 1
		echo "nitools++"

		##install the MCR 
		cd $TOOLS_PATH
		if [ ! -d MCR ]; then sudo -u $PL_USER mkdir MCR || exit 1; fi
		cd MCR || exit 1
		sudo -u $PL_USER unzip -o ${MCR_ARCHIVE_LOCATION} || exit 1
		echo "nitools+=2"
		sudo -u $PL_USER ./install       
    fi
	if [ "$INSTALL_MANUALTOOL_SERVERLIB" = "true" ]
	then
		if [ ! -d "$PL_SERVERLIB" ]
        then
            sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
        fi

        cd ${PL_SERVERLIB}
        echo "nitools+=3"
        sudo -u $PL_USER wget -c --progress=dot $server/serverLib_${MANUALTOOL_NAME}.tar.gz
        if [ $? -ne 0 ]; then
            echo "nitools!Failed to download $MANUALTOOL_NAME server library"
            exit 1
        fi

        echo "nitools+"
        sudo -u $PL_USER tar xmzvf serverLib_${MANUALTOOL_NAME}.tar.gz || exit 1
        if [ ! -z "${MANUALTOOL_VERSION}" ]
        then
			sudo -u $PL_USER sed -i "s/${MANUALTOOL_NAME}VERSION/${MANUALTOOL_VERSION}/g" ${PL_SERVERLIB}/*/*/*pipe
        fi
        echo "nitools+=2"
        sudo -u $PL_USER rm serverLib_${MANUALTOOL_NAME}.tar.gz || exit 1
        echo "nitools+"
        sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
	fi
else
    AT_LEAST_ONE_TOOL="false"

    if  [ "$INSTALL_AIR" = "true" ]
    then
		AT_LEAST_ONE_TOOL="true"
        echo "nitools=>13"
	fi
    if [ "$INSTALL_AIR_SERVERLIB" = "true" ]
    then
        echo "nitools=>7"
    fi

    if  [ "$INSTALL_BRAINSUITE" = "true" ]
    then
        AT_LEAST_ONE_TOOL="true"
        echo "nitools=>9"
	fi
    if [ "$INSTALL_BRAINSUITE_SERVERLIB" = "true" ]
    then
        echo "nitools=>7"
    fi

    if  [ "$INSTALL_AFNI" = "true" ]
    then
        AT_LEAST_ONE_TOOL="true"
        echo "nitools=>10"
	fi
    if [ "$INSTALL_AFNI_SERVERLIB" = "true" ]
    then
		echo "nitools=>7"
    fi

    if  [ "$INSTALL_FSL" = "true" ]
    then
        AT_LEAST_ONE_TOOL="true"
        echo "nitools=>9"
	fi
    if [ "$INSTALL_FSL_SERVERLIB" = "true" ]
    then
        echo "nitools=>7"
    fi
    
    if  [ "$INSTALL_FREESURFER" = "true" ]
    then
        AT_LEAST_ONE_TOOL="true"
        echo "nitools=>9"
	fi
    if [ "$INSTALL_FREESURFER_SERVERLIB" = "true" ]
    then
        echo "nitools=>7"
    fi
    
    if  [ "$INSTALL_LONITOOLS" = "true" ]
    then
        AT_LEAST_ONE_TOOL="true"
        echo "nitools=>130"
	fi
    if [ "$INSTALL_LONITOOLS_SERVERLIB" = "true" ]
	then
        echo "nitools=>7"
    fi

    if  [ "$INSTALL_MINC" = "true" ]
    then
        AT_LEAST_ONE_TOOL="true"
        echo "nitools=>8"
	fi
    if [ "$INSTALL_MINC_SERVERLIB" = "true" ]
    then
        echo "nitools=>7"
    fi
    
    if  [ "$INSTALL_ITK" = "true" ]
    then
        AT_LEAST_ONE_TOOL="true"
        echo "nitools=>20"
	fi
    if [ "$INSTALL_ITK_SERVERLIB" = "true" ]
    then
        echo "nitools=>7"
    fi

    if  [ "$INSTALL_GAMMA" = "true" ]
    then
        AT_LEAST_ONE_TOOL="true"
        echo "nitools=>10"
	fi
    if [ "$INSTALL_GAMMA_SERVERLIB" = "true" ]
    then
        echo "nitools=>7"
    fi

    if  [ "$INSTALL_DTK" = "true" ]
    then
        AT_LEAST_ONE_TOOL="true"
        echo "nitools=>9"
    fi
	if [ "$INSTALL_DTK_SERVERLIB" = "true" ]
    then
        echo "nitools=>7"
    fi

    if  [ "$INSTALL_SPM8" = "true" ]
    then
        AT_LEAST_ONE_TOOL="true"
        echo "nitools=>9"
    fi
	if [ "$INSTALL_SPM8_SERVERLIB" = "true" ]
    then
        echo "nitools=>7"
    fi

    if [ $AT_LEAST_ONE_TOOL = "false" ]
    then
        echo "nitools=>0"
    fi

    if [ ! -f "$TOOLS_PATH" ]
    then
        mkdir -p $TOOLS_PATH || exit 1
        chown $PL_USER $TOOLS_PATH
    fi

    cd $TOOLS_PATH

    ################################# FSL 7 steps
    if  [ "$INSTALL_FSL" = "true" ]
    then
        # if the archive string FSL_ARCHIVE_LOCATION for this manually-installed tool is not empty,
        # then we are in auto install mode, so we can install the executables and/or pipe files
            ######################################
            #######     INSTALLATION OF FSL
            ######################################
		if [ ! -z "$FSL_ARCHIVE_LOCATION" ]; then				
            sudo -u $PL_USER rm -Rf ${TOOLS_PATH}/FSL-${FSL_VERSION} || exit 1
            sudo -u $PL_USER tar xmvzf $FSL_ARCHIVE_LOCATION
            sudo -u $PL_USER mv fsl $TOOLS_PATH/FSL-${FSL_VERSION} || exit 1
            sudo -u $PL_USER ln -s $TOOLS_PATH/FSL-${FSL_VERSION} $TOOLS_PATH/FSL || exit 1

            # download and install glue and scripts
            cd $TOOLS_PATH/FSL-${FSL_VERSION}
            wget -c --progress=dot $server/fsl.tar.gz || { echo "nitools!Failed to download fsl glue"; exit 1; }
            sudo -u $PL_USER tar xmvzf fsl.tar.gz
            sudo -u $PL_USER rm fsl.tar.gz

			wget -c --progress=dot $server/fsl-scripts.tar.gz || { echo "nitools!Failed to download fsl scripts"; exit 1; }
			sudo -u $PL_USER tar xmvzf fsl-scripts.tar.gz
            sudo -u $PL_USER rm fsl-scripts.tar.gz
		else
		   echo 'nitools@FSL@http://fsl.fmrib.ox.ac.uk/fsldownloads@<b>After your browser opens, follow these instructions:</b><br>a. Click the "I agree to the below terms and conditions..." link.<br>b. Fill out your personal information and scroll down to the Advanced users section.<br>c. Select the "Linux CentOS5 64-bit" or "Linux CentOS6 64-bit"<br>d. Click the Download button and save the archive to your computer.<br>'
		fi
	fi
    if [ "$INSTALL_FSL_SERVERLIB" = "true" ]; then
		echo "nitools-->Installing: "
        echo "nitools->FSL server library files"
        if [ ! -d "$PL_SERVERLIB" ]; then
            sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
        fi

        cd ${PL_SERVERLIB}
        echo "nitools+=3"
        sudo -u $PL_USER wget -c --progress=dot $server/serverLib_FSL.tar.gz
        if [ $? -ne 0 ]; then
            echo "nitools!Failed to download FSL server library"
            exit 1
        fi

        echo "nitools++"
        sudo -u $PL_USER tar -xmzvf serverLib_FSL.tar.gz || exit 1
        if [ ! -z "${FSL_VERSION}" ]; then
            sudo -u $PL_USER sed -i "s/FSLVERSION/${FSL_VERSION}/g" ${PL_SERVERLIB}/*/*/*pipe
        fi
        echo "nitools+=2"
        sudo -u $PL_USER rm serverLib_FSL.tar.gz || exit 1
        echo "nitools++"
        sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
    fi
        
    cd $TOOLS_PATH || exit 1

    ################################# FreeSurfer 7 steps
    if  [ "$INSTALL_FREESURFER" = "true" ]
    then
        # if the archive string FREESURFER_ARCHIVE_LOCATION for this manually-installed tool is not empty,
        # then we are in auto install mode, so we can install the executables and/or pipe files
            ######################################
            #######     INSTALLATION OF FREESURFER
            ######################################
		if [ ! -z "$FREESURFER_ARCHIVE_LOCATION" ]; then
			sudo -u $PL_USER rm -Rf ${TOOLS_PATH}/FREESURFER-${FREESURFER_VERSION} || exit 1
			sudo -u $PL_USER tar xmvzf $FREESURFER_ARCHIVE_LOCATION
			sudo -u $PL_USER mv freesurfer $TOOLS_PATH/FREESURFER-${FREESURFER_VERSION} || exit 1
			sudo -u $PL_USER ln -s $TOOLS_PATH/FREESURFER-${FREESURFER_VERSION} $TOOLS_PATH/FREESURFER || exit 1

			# download and install glue and scripts
			cd $TOOLS_PATH/FREESURFER-${FREESURFER_VERSION}
			wget -c --progress=dot $server/freesurfer.tar.gz || { echo "nitools!Failed to download freesurfer glue"; exit 1; }
			sudo -u $PL_USER tar xmvzf freesurfer.tar.gz
			sudo -u $PL_USER rm freesurfer.tar.gz
		
			wget -c --progress=dot $server/freesurfer-scripts.tar.gz || { echo "nitools!Failed to download freesurfer scripts"; exit 1; }
			sudo -u $PL_USER tar xmvzf freesurfer-scripts.tar.gz
			sudo -u $PL_USER rm freesurfer-scripts.tar.gz

			if [ ! -z "$FREESURFER_LICENSE_LOCATION" ]; then
				cp $FREESURFER_LICENSE_LOCATION ${TOOLS_PATH}/FREESURFER-${FREESURFER_VERSION} || exit 1
			fi
		else
			echo 'nitools@FreeSurfer@http://surfer.nmr.mgh.harvard.edu/fswiki/Download@<b>After your browser opens, follow these instructions:</b><br>a. Click on freesurfer-Linux-centos4_x86_64-stable-pub-v5.0.0.tar.gz link.<br>b. Save the archive to your computer.<br>@https://surfer.nmr.mgh.harvard.edu/registration.html@<b>After your browser opens, follow these instructions:</b><br>a. Fill out the form on the FreeSurfer registration page<br>b. Make sure to select the proper OS<br>c. Press the "I AGREE" button at the bottom of the page<br>d. You will receive an email with instructions on how to create the .license file'
		fi
	fi
	if [ "$INSTALL_FREESURFER_SERVERLIB" = "true" ]; then
		echo "nitools-->Installing: "
        echo "nitools->FreeSurfer server library files"
		if [ ! -d "$PL_SERVERLIB" ]; then
			sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
		fi

		cd ${PL_SERVERLIB}
		echo "nitools+=3"
		sudo -u $PL_USER wget -c --progress=dot $server/serverLib_FreeSurfer.tar.gz
		if [ $? -ne 0 ]; then
			echo "nitools!Failed to download FreeSurfer server library"
			exit 1
		fi
		echo "nitools++"
		sudo -u $PL_USER tar -xmzvf serverLib_FreeSurfer.tar.gz || exit 1
		if [ ! -z "${FREESURFER_VERSION}" ]; then
			sudo -u $PL_USER sed -i "s/FREESURFERVERSION/${FREESURFER_VERSION}/g" ${PL_SERVERLIB}/*/*/*pipe
		fi
		echo "nitools+=2"
		sudo -u $PL_USER rm serverLib_FreeSurfer.tar.gz || exit 1
		echo "nitools++"
		sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
	fi
	
    cd $TOOLS_PATH || exit 1

    ################################# DTK 7 steps
    if  [ "$INSTALL_DTK" = "true" ]
    then
        # if the archive string DTK_ARCHIVE_LOCATION for this manually-installed tool is not empty,
        # then we are in auto install mode, so we can install the executables and/or pipe files
            #############################################
            #######     INSTALLATION OF DIFFUSION TOOLKIT
            #############################################
		if [ ! -z "$DTK_ARCHIVE_LOCATION" ]; then
			sudo -u $PL_USER rm -Rf ${TOOLS_PATH}/DTK-${DTK_VERSION} || exit 1
			sudo -u $PL_USER tar xmvzf $DTK_ARCHIVE_LOCATION
			sudo -u $PL_USER mv dtk $TOOLS_PATH/DTK-${DTK_VERSION} || exit 1
			sudo -u $PL_USER ln -s $TOOLS_PATH/DTK-${DTK_VERSION} $TOOLS_PATH/DTK || exit 1

			# download and install glue
			cd $TOOLS_PATH/DTK-${DTK_VERSION}
			wget -c --progress=dot $server/dtk.tar.gz
			if [ $? -ne 0 ]; then
				echo "nitools!Failed to download diffusion toolkit glue"
				exit 1
			fi
			sudo -u $PL_USER tar xmvzf dtk.tar.gz
			sudo -u $PL_USER rm dtk.tar.gz
		else
			echo 'nitools@DTK@http://trackvis.org/download/@<b>After your browser opens, follow these instructions:</b><br>1. Sign up or, if you have already registered, log in.<br>2. Select "Linux (64-bit)" as the platform from the drop down menu.<br>3. Press the Download button.<br>'
		fi
	fi
    if [ "$INSTALL_DTK_SERVERLIB" = "true" ]; then
		echo "nitools-->Installing: "
        echo "nitools->DTK server library files"
        if [ ! -d "$PL_SERVERLIB" ]; then
            sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
        fi

        cd ${PL_SERVERLIB}
        echo "nitools+=3"
        sudo -u $PL_USER wget -c --progress=dot $server/serverLib_DTK.tar.gz
        if [ $? -ne 0 ]; then
            echo "nitools!Failed to download DTK server library"
            exit 1
        fi

        echo "nitools++"
        sudo -u $PL_USER tar xmzvf serverLib_DTK.tar.gz || exit 1
        if [ ! -z "${DTK_VERSION}" ]; then
            sudo -u $PL_USER sed -i "s/DTKVERSION/${DTK_VERSION}/g" ${PL_SERVERLIB}/*/*/*pipe
        fi
        echo "nitools+=2"
        sudo -u $PL_USER rm serverLib_DTK.tar.gz || exit 1
        echo "nitools++"
        sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
    fi
	
    cd $TOOLS_PATH || exit 1
	
	    ################################# SPM 10 steps
    if  [ "$INSTALL_SPM8" = "true" ]; then
		if [ ! -z $MCR_ARCHIVE_LOCATION ]; then 
            echo "nitools-->Downloading: "
            echo "nitools->SPM v.${SPM_VERSION}"
	      
	        echo "nitools++"
            wget -c --progress=dot $server/spm8-${SPM_VERSION}.tar.gz
              
	        if [ $? -ne 0 ]; then
                echo "nitools!Failed to download SPM binaries"
                 exit 1
            fi

            echo "nitools+=3"

            echo "nitools-->Installing: "
            echo "nitools->SPM v.${SPM_VERSION}"

            echo "nitools++"
            sudo -u $PL_USER tar xmvzf spm8-${SPM_VERSION}.tar.gz || exit 1
            echo "nitools+=3"
            rm -f spm8-${SPM_VERSION}.tar.gz || exit 1
            echo "nitools++"
		else 
		   echo 'nitools@SPM8@http://www.mathworks.com/products/compiler/mcr/@<b>After your browser opens, follow these instructions:</b><br>1. Select the 64-bit Linux download link of version R2012a (7.17) of the MCR.<br>2. Save the file to your machine and continue to Step 2.@MCR'
        fi
	fi
	if [ "$INSTALL_SPM8_SERVERLIB" = "true" ]; then
        echo "nitools-->Installing: "
        echo "nitools->SPM server library files"
        if [ ! -d "$PL_SERVERLIB" ]; then
            sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
        fi

        cd ${PL_SERVERLIB} || exit 1
        echo "nitools++"
        sudo -u $PL_USER wget -c --progress=dot $server/serverLib_SPM8.tar.gz
        if [ $? -ne 0 ]; then
            echo "nitools!Failed to download SPM server library"
            exit 1
        fi

        echo "nitools+=3"
        sudo -u $PL_USER tar -xmzvf serverLib_SPM8.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER rm serverLib_SPM8.tar.gz || exit 1
        echo "nitools++"
        sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
    fi
    
    cd $TOOLS_PATH || exit 1

    ################################# BrainSuite 7 steps
    if  [ "$INSTALL_BRAINSUITE" = "true" ]; then
        wget -c --progress=dot $server/brainsuite11a.tar.gz
		if [ $? -ne 0 ]; then
            echo "nitools!Failed to download BRAINSUITE binaries"
            exit 1
        fi
	
        brainsuite_tar="brainsuite11a.tar.gz"
        echo "nitools++"
        echo "nitools-->Installing: "
        echo "nitools->Brainsuite v.${BRAINSUITE_VERSION}"
	
		sudo -u $PL_USER rm -Rf ${TOOLS_PATH}/BrainSuite-${BRAINSUITE_VERSION} || exit 1
		BRAINSUITE_ARCHIVE_LOCATION=$TOOLS_PATH/$brainsuite_tar

		echo "nitools++"

        sudo -u $PL_USER tar xmvzf $BRAINSUITE_ARCHIVE_LOCATION || exit 1
        echo "nitools+=3"
        sudo -u $PL_USER ln -s $TOOLS_PATH/BrainSuite-${BRAINSUITE_VERSION} $TOOLS_PATH/BRAINSUITE || exit 1		
        sudo -u $PL_USER rm -f $BRAINSUITE_ARCHIVE_LOCATION || exit 1	
        echo "nitools+=2"

        # download and install glue
        cd $TOOLS_PATH/BrainSuite-${BRAINSUITE_VERSION}
        wget -c --progress=dot $server/brainsuite_data.tar.gz
        if [ $? -ne 0 ]; then
            echo "nitools!Failed to download brainsuite glue"
            exit 1
        fi
        sudo -u $PL_USER tar xmvzf brainsuite_data.tar.gz
        echo "nitools++"
        sudo -u $PL_USER rm brainsuite_data.tar.gz
        echo "nitools++"
    fi
            
    if [ "$INSTALL_BRAINSUITE_SERVERLIB" = "true" ]; then
		echo "nitools-->Installing: "
        echo "nitools->Brainsuite server library files"
        if [ ! -d "$PL_SERVERLIB" ]; then
            sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
        fi

        cd ${PL_SERVERLIB} || exit 1
        sudo -u $PL_USER wget -c --progress=dot $server/serverLib_BrainSuite.tar.gz
        if [ $? -ne 0 ]; then
            echo "nitools!Failed to download brainsuite server library"
            exit 1
        fi

        echo "nitools++"
        sudo -u $PL_USER tar -xmzvf serverLib_BrainSuite.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER rm serverLib_BrainSuite.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1i
        echo "nitools+=2"
    fi
    
    cd $TOOLS_PATH || exit 1

    ################################# AIR 13 steps
    if  [ "$INSTALL_AIR" = "true" ]; then
        echo "nitools-->Downloading: "
        echo "nitools->AIR v.${AIR_VERSION}"
        ######################################
        #######     INSTALLATION OF AIR
        ######################################

        rm -Rf ${TOOLS_PATH}/AIR-${AIR_VERSION} || exit 1
        echo "nitools++"
        wget -c --progress=dot $server/air-${AIR_VERSION}.tar.gz
        if [ $? -ne 0 ]; then
			echo "nitools!Failed to download AIR binaries"
 	        exit 1
        fi

        echo "nitools+=8"
        echo "nitools-->Installing: "
        echo "nitools->AIR v.${AIR_VERSION}"
        
        sudo -u $PL_USER tar xmvzf air-${AIR_VERSION}.tar.gz || exit 1
        rm -f air-${AIR_VERSION}.tar.gz || exit 1
        echo "nitools+=4"
    fi

    if [ "$INSTALL_AIR_SERVERLIB" = "true" ]; then
        echo "nitools-->Installing: "
        echo "nitools->AIR server library files"
	    if [ ! -d "$PL_SERVERLIB" ]; then
            sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
        fi

        cd $PL_SERVERLIB || exit 1
        echo "nitools+=3"
        sudo -u $PL_USER wget -c --progress=dot $server/serverLib_AIR.tar.gz
        if [ $? -ne 0 ]; then
			echo "nitools!Failed to download AIR server library"
 	        exit 1
        fi

        echo "nitools++"
        sudo -u $PL_USER tar -xmzvf serverLib_AIR.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER rm serverLib_AIR.tar.gz || exit 1
        echo "nitools++"
        sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
    fi
   
    cd $TOOLS_PATH || exit 1

    ################################# AFNI 10 steps
    if  [ "$INSTALL_AFNI" = "true" ]; then
        echo "nitools-->Installing: "
        echo "nitools->libXp for AFNI"
        yum -y install libXp
        echo "nitools+=2"

        echo "nitools-->Downloading: "
        echo "nitools->AFNI v.${AFNI_VERSION}"

        wget -c --progress=dot $server/afni-${AFNI_VERSION}.tar.gz
        if [ $? -ne 0 ]; then
	        echo "nitools!Failed to download AFNI binaries"
 	        exit 1
        fi
        
        echo "nitools+=5"
        echo "nitools-->Installing: "
        echo "nitools->AFNI v.${AFNI_VERSION}"
        sudo -u $PL_USER tar xmvzf afni-${AFNI_VERSION}.tar.gz || exit 1
        echo "nitools+=2"
        rm -f afni-${AFNI_VERSION}.tar.gz || exit 1
        echo "nitools++"
    fi

    if [ "$INSTALL_AFNI_SERVERLIB" = "true" ]; then
        echo "nitools-->Installing: "
        echo "nitools->AFNI server library files"
	    if [ ! -d "$PL_SERVERLIB" ]; then
            sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
        fi

		cd ${PL_SERVERLIB} || exit 1
        echo "nitools+=3"
        sudo -u $PL_USER wget -c --progress=dot $server/serverLib_AFNI.tar.gz
        if [ $? -ne 0 ]; then
 	        echo "nitools!Failed to download AFNI server library"
 	        exit 1
        fi

		echo "nitools++"
        sudo -u $PL_USER tar -xmzvf serverLib_AFNI.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER rm serverLib_AFNI.tar.gz || exit 1
        echo "nitools++"
        sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
    fi
    
    cd $TOOLS_PATH || exit 1

    ################################# MINC 6 steps
    if  [ "$INSTALL_MINC" = "true" ]; then
        echo "nitools-->Downloading: "
        echo "nitools->MINC v.${MINC_VERSION}"
        wget -c --progress=dot $server/minc-${MINC_VERSION}.tar.gz
        if [ $? -ne 0 ]; then
 	        echo "nitools!Failed to download MINC binaries"
 	        exit 1
        fi

        echo "nitools+=5"
        echo "nitools-->Installing: "
        echo "nitools->MINC v.${MINC_VERSION}"

        sudo -u $PL_USER tar xmvzf minc-${MINC_VERSION}.tar.gz || exit 1
        echo "nitools+=2"
        rm -f minc-${MINC_VERSION}.tar.gz || exit 1
        echo "nitools++"
    fi

    if [ "$INSTALL_MINC_SERVERLIB" = "true" ]; then
        echo "nitools-->Installing: "
        echo "nitools->MINC server library files"
        if [ ! -d "$PL_SERVERLIB" ]; then
            sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
        fi

        cd ${PL_SERVERLIB} || exit 1
        echo "nitools+=3"
        sudo -u $PL_USER wget -c --progress=dot $server/serverLib_MINC.tar.gz
        if [ $? -ne 0 ]; then
			echo "nitools!Failed to download MINC server library"
 	        exit 1
        fi

        echo "nitools++"
        sudo -u $PL_USER tar -xmzvf serverLib_MINC.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER rm serverLib_MINC.tar.gz || exit 1
        echo "nitools++"
        sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
    fi
    
    cd $TOOLS_PATH || exit 1

    ################################# ITK 20 steps
    if  [ "$INSTALL_ITK" = "true" ]; then
        echo "nitools-->Downloading: "
        echo "nitools->ITK v.${ITK_VERSION}"
        wget -c --progress=dot $server/itk-${ITK_VERSION}.tar.gz
        if [ $? -ne 0 ]; then
	        echo "nitools!Failed to download ITK binaries"
 	        exit 1
        fi

        echo "nitools+=10"
        echo "nitools-->Installing: "
        echo "nitools->ITK v.${ITK_VERSION}"

        echo "nitools+=4"
        sudo -u $PL_USER tar xmvzf itk-${ITK_VERSION}.tar.gz || exit 1
        echo "nitools+=4"
        rm -f itk-${ITK_VERSION}.tar.gz || exit 1
        echo "nitools+=2"
    fi

    if [ "$INSTALL_ITK_SERVERLIB" = "true" ]; then
	    echo "nitools-->Installing: "
        echo "nitools->ITK v.${ITK_VERSION} server library files"
	    if [ ! -d "$PL_SERVERLIB" ]; then
            sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
        fi

        cd ${PL_SERVERLIB} || exit 1
        echo "nitools+=3"

        sudo -u $PL_USER wget -c --progress=dot $server/serverLib_ITK.tar.gz
        if [ $? -ne 0 ]; then
			echo "nitools!Failed to download ITK server library"
 	        exit 1
        fi

        echo "nitools++"
        sudo -u $PL_USER tar -xmzvf serverLib_ITK.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER rm serverLib_ITK.tar.gz || exit 1
        echo "nitools++"
        sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
    fi
    
    cd $TOOLS_PATH || exit 1

    ################################# GAMMA 10 steps
    if  [ "$INSTALL_GAMMA" = "true" ]; then
        echo "nitools-->Downloading: "
        echo "nitools->GAMMA v.${GAMMA_VERSION}"

        wget -c --progress=dot $server/gamma-${GAMMA_VERSION}.tar.gz
        if [ $? -ne 0 ]; then
 	        echo "nitools!Failed to download GAMMA binaries"
 	        exit 1
        fi
        echo "nitools+=5"
        echo "nitools-->Installing: "
        echo "nitools->GAMMA v.${GAMMA_VERSION}"
            
        echo "nitools+=3"
        sudo -u $PL_USER tar xmvzf gamma-${GAMMA_VERSION}.tar.gz || exit 1
        echo "nitools++"
        rm -f gamma-${GAMMA_VERSION}.tar.gz || exit 1
        echo "nitools++"
    fi

    if [ "$INSTALL_GAMMA_SERVERLIB" = "true" ]; then
        echo "nitools-->Installing: "
        echo "nitools->GAMMA server library files"
        if [ ! -d "$PL_SERVERLIB" ]; then
            sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
        fi

        cd ${PL_SERVERLIB} || exit 1
        echo "nitools+=3"
        sudo -u $PL_USER wget -c --progress=dot $server/serverLib_GAMMA.tar.gz
        if [ $? -ne 0 ]; then
 	        echo "nitools!Failed to download GAMMA server library"
 	        exit 1
        fi

        echo "nitools++"
        sudo -u $PL_USER tar -xmzvf serverLib_GAMMA.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER rm serverLib_GAMMA.tar.gz || exit 1
        echo "nitools++"
        sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
    fi
    
    cd $TOOLS_PATH || exit 1

    ################################# LONI TOOLS 130 steps
    if  [ "$INSTALL_LONITOOLS" = "true" ]; then
        echo "nitools-->Cleaning previous installations of LONI Tools..."
        echo "nitools->"
        ######################################
        #######     INSTALLATION OF LONI TOOLS
        ######################################
        rm -Rf ${TOOLS_PATH}/LONI || exit 1

        sudo -u $PL_USER mkdir -p $TOOLS_PATH/LONI || exit 1
        sudo -u $PL_USER mkdir -p $TOOLS_PATH/LONI/data || exit 1
        sudo -u $PL_USER mkdir -p $TOOLS_PATH/LONI/scripts || exit 1
        sudo -u $PL_USER mkdir -p $TOOLS_PATH/LONI/apps || exit 1
        sudo -u $PL_USER mkdir -p $TOOLS_PATH/LONI/workflows || exit 1
        sudo -u $PL_USER mkdir -p $TOOLS_PATH/LONI/jars || exit 1
        
        echo "nitools++"
        echo "nitools-->Downloading: "
        echo "nitools->Jar files of LONI Tools [ 1/5 ]"

        wget -c --progress=dot $server/loniJars.tar.gz
        if [ $? -ne 0 ]; then
			echo "nitools!Failed to download LONI jars"
 	        exit 1
        fi
        echo "nitools+=3"
        echo "nitools-->Downloading: "
        echo "nitools->Scripts of LONI Tools [ 2/5 ]"

        wget -c --progress=dot $server/loniScripts.tar.gz
        if [ $? -ne 0 ]; then
 	        echo "nitools!Failed to download LONI scripts"
 	        exit 1
        fi
        
        echo "nitools++"
        echo "nitools-->Downloading: "
        echo "nitools->Apps of LONI Tools [ 3/5 ]"

        wget -c --progress=dot $server/loniApps.tar.gz
        if [ $? -ne 0 ]; then
 	        echo "nitools!Failed to download LONI apps"
 	        exit 1
        fi
        
        echo "nitools+=10"
        echo "nitools-->Downloading: "
        echo "nitools->Workflows of LONI Tools [ 4/5 ]"

        wget -c --progress=dot $server/loniWorkflows.tar.gz
        if [ $? -ne 0 ]; then
 	        echo "nitools!Failed to download brainsuite LONI workflows"
 	        exit 1
        fi
        
        echo "nitools+=5"
        echo "nitools-->Downloading: "
        echo "nitools->Data files of LONI Tools [ 5/5 ]"

        wget -c --progress=dot $server/loniData.tar.gz
        if [ $? -ne 0 ]; then
 	        echo "nitools!Failed to download LONI data"
 	        exit 1
        fi

        echo "nitools+=40"

        cd $TOOLS_PATH/LONI || exit 1
        echo "nitools-->Installing: "
        echo "nitools->[1/5] JAR files of LONI Tools"

        cd jars || exit 1
        mv $TOOLS_PATH/loniJars.tar.gz ./loniJars.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER tar xmvzf loniJars.tar.gz
        rm -f loniJars.tar.gz || exit 1
        echo "nitools+=3"

        echo "nitools-->Installing: "
        echo "nitools->[2/5] Scripts of LONI Tools"
        cd ../scripts || exit 1
        mv $TOOLS_PATH/loniScripts.tar.gz ./loniScripts.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER tar xmvzf loniScripts.tar.gz
        rm -f loniScripts.tar.gz || exit 1
        echo "nitools++"

        echo "nitools-->Installing: "
        echo "nitools->[3/5] Apps of LONI Tools"
        cd ../apps || exit 1
        mv $TOOLS_PATH/loniApps.tar.gz ./loniApps.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER tar xmvzf loniApps.tar.gz
        rm -f loniApps.tar.gz || exit 1
        echo "nitools+=6"

        echo "nitools-->Installing: "
        echo "nitools->[4/5] Workflows of LONI Tools"
        cd ../workflows || exit 1
        mv $TOOLS_PATH/loniWorkflows.tar.gz ./loniWorkflows.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER tar xmvzf loniWorkflows.tar.gz
        rm -f loniWorkflows.tar.gz || exit 1
        sudo -u $PL_USER ln -s $TOOLS_PATH/LONI/workflows $TOOLS_PATH/loniWorkflows || exit 1
	    echo "nitools+=15"

        echo "nitools-->Installing: "
        echo "nitools->[5/5] Data files of LONI Tools"
        cd ../data || exit 1
        mv $TOOLS_PATH/loniData.tar.gz ./loniData.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER tar xmvzf loniData.tar.gz
        rm -f loniData.tar.gz || exit 1
        echo "nitools+=35"
    fi

    if [ "$INSTALL_LONITOOLS_SERVERLIB" = "true" ]; then
        if [ ! -d "$PL_SERVERLIB" ]; then
            sudo -u $PL_USER mkdir -p $PL_SERVERLIB || exit 1
        fi
            
        cd ${PL_SERVERLIB} || exit 1
        echo "nitools+=3"
        sudo -u $PL_USER wget -c --progress=dot $server/serverLib_LONI.tar.gz
        if [ $? -ne 0 ]; then
			echo "nitools!Failed to download LONI server library"
 	        exit 1
        fi

        echo "nitools++"
		sudo -u $PL_USER tar xmzvf serverLib_LONI.tar.gz || exit 1
        echo "nitools+=2"
        sudo -u $PL_USER rm serverLib_LONI.tar.gz || exit 1
        echo "nitools++"
        sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
    fi
fi

running_tools=$(ps aux | grep "tar xm" | cut -d'/' -f4 | cut -d'-' -f1 | sed 's/\n//g')
if [ ! -z $running_tools ]; then
   echo "nitools!-->Finishing install of: "
   echo "nitools!->$running_tools"
fi

# update hostname specified in module definitions
HOSTNAME=${PL_HOSTNAME}
if [ "${PL_PORT}" != 8001 ]; then
	HOSTNAME=${PL_HOSTNAME}:${PL_PORT}
fi

if [ -d "${PL_SERVERLIB}" ]; then
	for x in `ls ${PL_SERVERLIB}/*/*/*pipe 2> /dev/null`; do
                if [ -f $x ]
                then  
		   sed -i "s/cranium\.loni\.usc\.edu/$HOSTNAME/g" $x || exit 1
                fi
	done
	sudo -u $PL_USER touch ${PL_SERVERLIB}/.monitorFile || exit 1
fi

