#!/bin/bash

redhat=false
debian=false
#check redhat/centos/fedora
if [ -a /etc/redhat-release ]; then
    ssh="sshd"
    javainstall='rpm --prefix=$SHARED_FILESYSTEM_PATH -ivh $FILENAME'
    SGE_PORT=`grep sge_qmaster /etc/services | tr -s ' ' | sed -e 's/^[ \t]*//' | cut -d' ' -f2 | cut -d'/' -f1 | uniq`
    redhat=true
#check debian/ubuntu
elif [ "$(grep -is debian /etc/*-release)" ]; then 
    ssh="ssh"
    javainstall='tar --directory=$SHARED_FILESYSTEM_PATH -zxvf $FILENAME'
    SGE_PORT=`grep sge_qmaster /etc/services | cut -f2 | cut -d'/' -f1 |     uniq`
    debian=true
fi

if [ -z "$INSTALL_PIPELINE" ]
then
    echo "Please do not run this script manually. Run launchInstaller.sh instead."
    exit 1
fi

# add /usr/sbin and /sbin to PATH
export PATH=${PATH}:/usr/sbin:/sbin

DL_DIR=$(cd `dirname $0` && pwd)

if [ "$INSTALLER_MODE_MANUAL_TOOL" != "true" ]
then
    #echo "pl=>22"
    :;
fi

##################################################
#           CLEANING PREVIOUS INSTALLATIONS
##################################################
echo "=======| CLEANING PREVIOUS INSTALLATIONS |======="
echo "pl-->Cleaning previous installations..."
echo "pl->"

if [ -f /etc/init.d/pipeline ]
then
    /etc/init.d/pipeline stop
fi

# before removing the Pipeline directory, move the server library files so that
# these files are not lost; we will copy them back after the installation
# check if the servlibrary directory exists and that its not empty
if [ -d ${PL_LOCATION}/serverLibrary ]
then
   if [ $(ls -l $PL_LOCATION/serverLibrary | head -1 | cut -d' ' -f2) != 0 ]
   then
      sudo -u $PL_USER mkdir -p /tmp/serverLibrary || { echo "Error while writing to /tmp." && exit 1; }
      sudo -u $PL_USER mv ${PL_LOCATION}/serverLibrary/* /tmp/serverLibrary/
   fi
fi

if [ -d ${PL_LOCATION} ]
then
    rm -rf $(ls -d $PL_LOCATION/* | grep -v ${PL_LOCATION}/serverLibrary) $PLDB_LOCATION
fi

if [ "$INSTALLER_MODE_MANUAL_TOOL" != "true" ]
then
echo "pl++"
fi

##################################################
#           CREATING INSTALLATION DIRECTORIES
##################################################
echo "=======| CLEANING INSTALLATION DIRECTORIES |======="
mkdir -p $PL_LOCATION/dist
chown $PL_USER $PL_LOCATION
chown $PL_USER $PL_LOCATION/dist

mkdir -p $PLDB_LOCATION
chown $PL_USER $PLDB_LOCATION

mkdir -p $PL_TEMPDIR
chown $PL_USER $PL_TEMPDIR

mkdir -p $PL_SCRATCHDIR
chown $PL_USER $PL_SCRATCHDIR

if [ -d /tmp/serverLibrary ]
then
    sudo -u $PL_USER cp -ru /tmp/serverLibrary ${PL_LOCATION}
    sudo -u $PL_USER rm -r /tmp/serverLibrary
fi

mkdir -p $TOOLS_PATH
chown $PL_USER $TOOLS_PATH

##################################################
#           JDK INSTALLATION
##################################################
# echo "=======| CHECKING FOR ORACLE JDK |======="
# echo "pl-->Checking for Oracle JDK..."
# echo "pl->"
if [ "$INSTALLER_MODE_MANUAL_CANCEL" != "true" ]; then
if [ `java -version 2>&1 | grep 'Java(TM)' | wc -l` -ne 1 ]
then
    if [ ! -z "$JDK_BINARY_LOCATION" ]
    then
        INSTALLER_MODE_MANUAL_TOOL=true
        MANUALTOOL_ARCHIVE_PATH=$JDK_BINARY_LOCATION
    fi
    if [ "$INSTALLER_MODE_MANUAL_TOOL" = "true" ]
    then
        echo "The provided JDK archive path is: $MANUALTOOL_ARCHIVE_PATH"
        echo "pl-->Installing: "
        echo "pl->Oracle JDK"
        cp "$MANUALTOOL_ARCHIVE_PATH" $SHARED_FILESYSTEM_PATH

        echo MANUALTOOL_ARCHIVE_PATH=$MANUALTOOL_ARCHIVE_PATH
        echo SHARED_FILESYSTEM_PATH=$SHARED_FILESYSTEM_PATH
        FILENAME=${MANUALTOOL_ARCHIVE_PATH##*/}

        # install jdk rpm
        echo FILENAME=$FILENAME
        cd $SHARED_FILESYSTEM_PATH
	    
		if [ ! -f /usr/sbin/alternatives ]; then
		    echo "Linking update-alternatives and alternatives"
			ln -s /usr/sbin/update-alternatives /usr/sbin/alternatives
		fi
        eval $javainstall 2> javaerrors.log
		if [ "`grep already ./javaerrors.log`" ]; then
			rm ./javaerrors.log
			echo "---------------------------------------------------------------------------------------------------------------"
			echo "Oracle Java package is already installed. Set it to default using the update-alternatives command"
			echo "---------------------------------------------------------------------------------------------------------------"
		fi
        rm -f $FILENAME   

		# determine exact path where jdk was installed
		javapath=`ls -d $SHARED_FILESYSTEM_PATH/jdk*`
		if [ `echo $javapath | wc -w` -ne 1 ]
		then
			javapath=`echo $javapath | cut -d' ' -f1`
		fi
		# verify that javapath is pointing to the correct location
		${javapath}/bin/java -version > /dev/null 2> /dev/null
		if [ $? == 0 ]; then
			update-alternatives --install /usr/bin/java java ${javapath}/bin/java 2
			update-alternatives --set java ${javapath}/bin/java
			JAVA_HOME=${javapath}

			export JAVA_HOME
			echo JAVA_HOME=$JAVA_HOME

			# configure JAVA_HOME in /etc/profile and /etc/bash.bashrc if they exist
			for file in /etc/profile /etc/bash.bashrc; do
				if [ -f $file ] && [ -z "`grep JAVA_CONFIG $file`" ]
				then
				    echo " " >> $file
					echo -e "########## JAVA_CONFIG\nexport JAVA_HOME=$JAVA_HOME\n" >> $file
				else
					javahome_lineNumber=$(grep -n "export JAVA_HOME" $file | cut -d':' -f1)
					sed -i ${javahome_lineNumber}c"export JAVA_HOME=$JAVA_HOME" $file
				fi
			done
			
			if [ -f /etc/csh.login ]; then
				if [ -z "`grep JAVA_CONFIG /etc/csh.login`" ]
				then
				    echo " " >> /etc/csh.login
					echo -e "########## JAVA_CONFIG\nsetenv JAVA_HOME $JAVA_HOME\n">> /etc/csh.login
				else
					javahome_lineNumber=$(grep -n "setenv JAVA_HOME" /etc/csh.login | cut -d':' -f1)
					sed -i ${javahome_lineNumber}c"setenv JAVA_HOME $JAVA_HOME" /etc/csh.login
				fi
			fi

			echo "JDK Installed"
		fi
	else
	    if [ "$redhat" = "true" ]; then
			echo 'pl@Oracle JDK@http://www.oracle.com/technetwork/java/javase/downloads/index.html@1. Click on Download JDK button.<br>2. Select Platform: Linux x64 or x86 and click Continue ( username, password is optional )<br>3. Click on jdk-[ver]-linux-[platform].rpm link to begin download.'
        else 
		    echo 'pl@Oracle JDK@http://www.oracle.com/technetwork/java/javase/downloads/index.html@1. Click on Download JDK button.<br>2. Select Platform: Linux x64 or x86 and click Continue ( username, password is optional )<br>3. Click on jdk-[ver]-linux-[platform].tar.gz link to begin download.'
		fi
		exit 0;
		
    fi
fi
fi

# Source /etc/profile
# . /etc/profile

# If we get here, you have an official version of java, set JAVA_HOME
# First try setting variables with alternatives command
for version in `update-alternatives --display java | grep priority | cut -d' ' -f1`; do 
   if [ "$($version -version 2>&1 | grep "Java(TM)")" ]; then 
      echo $version    
      JAVA_BINARY=$version
      JAVA_BIN=`dirname $JAVA_BINARY`
      JAVA_HOME=`dirname $JAVA_BIN`
      export JAVA_HOME
      break
   fi
done
# If alternatives doesn't work, there is only 1 official version
if [ -z "$JAVA_HOME" ]
then
   #used to be readlink $(which java) but that sometimes led to /etc/bin/java which doesn't exist
   JAVA_BINARY=`which java`
   JAVA_BIN=`dirname $JAVA_BINARY`
   JAVA_HOME=`dirname $JAVA_BIN`
   export JAVA_HOME
fi

echo "pl+=2"

##################################################
#           DOWNLOADING
##################################################
echo "=======| DOWNLOADING FILES |======="
if [ -d ${DL_DIR}/pipeline ]
then
    cd ${DL_DIR}/pipeline
else
    mkdir -p ${DL_DIR}/pipeline
    cd ${DL_DIR}/pipeline
fi

echo "pl-->Downloading: "
echo "pl->Pipeline package"
wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/Pipeline.tar.bz2 || { echo "Error while running wget." && exit 1; }
echo "pl+=2"
wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/drmaa_plugin.tar.gz
echo "pl++"
wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/jgdi_plugin.tar.gz
echo "pl++"
wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/plgridplugin.tar.gz
echo "pl++"
wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/plgridservice.tar.gz

echo "pl+=3"


echo "pl-->Downloading: "
echo "pl->Authentication module library"
wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/libjaaspam.so

echo "pl+=2"

echo "pl-->Downloading: "
echo "pl->Authentication module"

if [ "$PL_USER_AUTH" = "SSH" ]
then
    wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/SSHLocalLoginModule.tar.gz
    tar xmvzf SSHLocalLoginModule.tar.gz
    LOGIN_MODULE_CLASS="edu.ucla.loni.pipeline.security.SSHLocalLoginModule"
    LOGIN_MODULE=':$BASEPATH/dist/lib/SSHLocalLoginModule.jar'
    LOGIN_MODULE_FILE="SSHLocalLoginModule.jar"
    service $ssh restart || { echo "Error when starting sshd service." && exit 1; }
elif [ "$PL_USER_AUTH" = "PAM" ]
then
    wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/PAMLoginModule.jar
    LOGIN_MODULE_CLASS="edu.ucla.loni.pipeline.security.LONILoginModule"
    LOGIN_MODULE=':$BASEPATH/dist/lib/PAMLoginModule.jar'
    LOGIN_MODULE_FILE="PAMLoginModule.jar"
else
    echo "No authentication module to download. User selected no authentication. "
    LOGIN_MODULE_CLASS="com.sun.security.auth.module.UnixLoginModule"
    LOGIN_MODULE=""
    LOGIN_MODULE_FILE=""
fi

echo "pl+=2"

##################################################
#           EXTRACTING
##################################################
echo "=======| EXTRACTING FILES |======="
echo "pl-->Extracting: "
echo "pl->Pipeline package"
tar xmvjf Pipeline.tar.bz2
echo "pl++"





##################################################
#           INSTALLING PERSISTENCE DB
##################################################
echo "=======| INSTALLING PERSISTENCE DB |======="
echo "pl-->Installing: "
echo "pl->Persistence Database"

sudo -u $PL_USER cp lib/hsqldb.jar $PLDB_LOCATION/

###########SYSTEM ARCHITECTURE
# check if we are running on a 64-bit architecture
d64=
if java -d64 -version >/dev/null 2>&1; then
    d64=-d64
fi

###########MEMORY ALLOCATION
# check if the user has specified JVM memory allocation
memory=
if [ ! -z "$PL_MEMORY_ALLOC" ]
then
    memory=-Xmx${PL_MEMORY_ALLOC}M
fi

###########CREATING START DB SCRIPT
echo '
#! /bin/bash

DB_PATH="'$PLDB_LOCATION'"
SERVER_BASEPATH="'$PL_LOCATION'"

if [ -f ${DB_PATH}/running.pid ]
then
    echo "Failed to start database, seems there is already running database"
    exit 1;
fi

WHOAMI=`whoami`
if [ $WHOAMI = "root" ]
then
    echo "You are root. Please run this script only with user:'$PL_USER'"
    exit 1
fi

'$JAVA_HOME'/bin/java '$d64' -server '${memory}' -cp $DB_PATH/hsqldb.jar org.hsqldb.Server -database.1 file:$DB_PATH/persistenceDB'${PL_HOSTNAME}${PL_PORT}' -dbname.1 xdb1 -port 9002 > $DB_PATH/outputStream.log 2> $DB_PATH/errorStream.log&

if [ $? -eq 0 ]
then
	echo $! > ${DB_PATH}/running.pid
	echo "Database successfully started."
	exit 0
else
	echo "Failed to start database"
	exit 1
fi' >>$PLDB_LOCATION/startDB.sh


echo "pl++"

###########CREATING STOP DB SCRIPT
echo '#! /bin/bash
DB_PATH="'$PLDB_LOCATION'"
PID_FILE=${DB_PATH}/running.pid

if [ -f ${PID_FILE} ]
then
	echo -n "Killing database..."
	PID=`cat ${PID_FILE}`

	kill -9 $PID 2>/dev/null

	while ps -p ${PID} > /dev/null; do sleep 1; done

	if [ $? -eq 0 ]
	then
    		rm ${PID_FILE}
    		echo "DONE"
    		exit 0
	else
    		echo "FAILED"
    		exit 1
	fi
else
	echo "No database running."
fi'>>$PLDB_LOCATION/stopDB.sh


###### Changing start & stop script executable permissions

chown $PL_USER $PLDB_LOCATION/startDB.sh
chown $PL_USER $PLDB_LOCATION/stopDB.sh

chmod +x $PLDB_LOCATION/startDB.sh
chmod +x $PLDB_LOCATION/stopDB.sh

##################################################
#           INSTALLING THE SERVER
##################################################
echo "=======| INSTALLING PIPELINE SERVER |======="
echo "pl-->Installing: "
echo "pl->Pipeline Server"

firewall_file=/etc/sysconfig/system-config-securitylevel
if [ ! -f $firewall_file ]
then
    firewall_file=/etc/sysconfig/system-config-firewall
fi

if [ -f $firewall_file ]
then
    echo "Configuring firewall to allow for Pipeline communication. A backup of the previous configuration is stored in the /etc/syconfig/ directory."
    if [ -z "`grep enabled $firewall_file`" ]
    then
        # Firewall is Off,
        lokkit -q --selinux='disabled' --disabled --port=ssh:tcp --port=$PL_PORT:tcp
    else
        # Firewall is On,
        lokkit -q --selinux='disabled' --enabled --port=ssh:tcp --port=$PL_PORT:tcp
    fi
fi

if [ ! -z $SGE_ROOT ]
then
    export SGE_ROOT=$SGE_ROOT
fi

if [ ! -z $SGE_PORT ]
then
    export SGE_PORT=$SGE_PORT
else
    export SGE_PORT=6444
fi

echo "=======| Copying files"
mv lib $PL_LOCATION/dist/lib
mv Pipeline.jar $PL_LOCATION/dist/Pipeline.jar
mv libjaaspam.so $PL_LOCATION/dist/
mkdir -p $PL_LOCATION/dist/gridplugins
tar xmvzf drmaa_plugin.tar.gz
tar xmvzf jgdi_plugin.tar.gz
tar xmvzf plgridplugin.tar.gz
tar xmvzf plgridservice.tar.gz
mv DRMAAPlugin.jar $PL_LOCATION/dist/gridplugins/
mv JGDIPlugin.jar $PL_LOCATION/dist/gridplugins/
mv PipelineGridService.jar $PL_LOCATION/dist/gridplugins/
mv PipelineGridPlugin.jar $PL_LOCATION/dist/lib/
if [ "$LOGIN_MODULE_FILE" != "" ]
then
    cp $LOGIN_MODULE_FILE $PL_LOCATION/dist/lib/
fi

echo "=======| Changing ownership of files"
chown $PL_USER $PL_LOCATION/*
chown $PL_USER $PL_LOCATION/dist/*
chown $PL_USER $PL_LOCATION/dist/lib/*
chown $PL_USER $PL_LOCATION/dist/gridplugins/*

echo "=======| Checking selected grid plugin"
echo Selected plugin is
if [ "$PL_PLUGIN" = "DRMAA" ]
then
    echo DRMAA
    GRIDPLUGIN_USEARRAYJOBS=false
    GRIDPLUGIN_JARFILES=$PL_LOCATION/dist/gridplugins/DRMAAPlugin.jar
    GRIDPLUGIN_CLASS=drmaaplugin.DRMAAPlugin
elif [ "$PL_PLUGIN" = "JGDI" ]
then
    echo JGDI
    GRIDPLUGIN_USEARRAYJOBS=true
    GRIDPLUGIN_JARFILES="$PL_LOCATION/dist/gridplugins/JGDIPlugin.jar, $SGE_ROOT/lib/jgdi.jar, $PL_LOCATION/dist/lib/mysql-connector-java-5.1.8-bin.jar, $PL_LOCATION/dist/lib/hsqldb.jar"
    GRIDPLUGIN_CLASS=jgdiplugin.JGDIPlugin
else
    echo "[NONE]"
fi
echo "=======| Creating preferences file"

if [ ! -z "$GRIDPLUGIN_JARFILES" ]
then
    QUEUE_PREFS="
    <GridPluginClass>$GRIDPLUGIN_CLASS</GridPluginClass>
    <GridTotalSlotsCmd>qstat -g c | grep _pqueue | awk ""'"'{ print $6-$7-$8 }'"'""</GridTotalSlotsCmd>
    <GridEngineNativeSpecification>-shell y -S /bin/sh -w n -q _pqueue -b y _pmpi _pmem _pstack _pcomplex -N _pjob</GridEngineNativeSpecification>
    <GridPluginJARFiles>$GRIDPLUGIN_JARFILES</GridPluginJARFiles>
    <GridUseArrayJobs>$GRIDPLUGIN_USEARRAYJOBS</GridUseArrayJobs>
    <GridArrayJobsDynamicIncrease>$GRIDPLUGIN_USEARRAYJOBS</GridArrayJobsDynamicIncrease>
    <GridSubmissionQueue>$PL_QUEUE</GridSubmissionQueue>
    <GridPluginUseRestartableService>true</GridPluginUseRestartableService>"
fi

###########CREATING PREFERENCES FILE
## clear preferences file first line
echo "<?xml version="'"'1.0'"'" encoding="'"'UTF-8'"'"?>" > $PL_LOCATION/preferences.xml
echo "<preferences>
    <ServerPort>$PL_PORT</ServerPort>
    <Hostname>$PL_HOSTNAME</Hostname>
    <SecureTempFileLocation>$PL_TEMPDIR</SecureTempFileLocation>
    <UsePrivilegeEscalation>$PL_USEPRIVESC</UsePrivilegeEscalation>$QUEUE_PREFS
    <PersistenceURL>jdbc:hsqldb:hsql://localhost:9002/xdb1</PersistenceURL>
    <LogFileLocation>$PL_LOCATION/events.log</LogFileLocation>
    <ServerAdmins>$PL_USER</ServerAdmins>
    <ServerLibraryLocation>${PL_SERVERLIB}</ServerLibraryLocation>
    <ServerLibrarySameDirMonitor>false</ServerLibrarySameDirMonitor>
    <ServerLibraryMonitorFile>${PL_SERVERLIB}/.monitorFile</ServerLibraryMonitorFile>
</preferences>
">> $PL_LOCATION/preferences.xml
chown $PL_USER $PL_LOCATION/preferences.xml

##########ADDING UNIX COMMANDS TO USER DATA FILE
# check if user_data_file is set. If not, try to set it. If we can't, skip all this.
# if user_data_file is set, write to it as normal
if [ -z "${USER_DATA_FILE}" ]; then
   USER_DATA_FILE="$(/bin/sh -c "echo ~$PL_USER")/.pipeline/$PL_PORT/userdata.xml"
fi
if [ ! -z ${USER_DATA_FILE} ] && [ -a ${USER_DATA_FILE} ]; then
   lastline=$(cat ${USER_DATA_FILE} | grep -n "</userdata>" | cut -d':' -f1)
   if [ "$lastline" = "" ]; then
      userdataline=$(cat ${USER_DATA_FILE} | grep -n "<userdata />" | cut -d':' -f1)
      addone=$(echo ${userdataline} + 1 | bc -l)
      sed -i 's/<userdata \/>/<userdata>/g' $USER_DATA_FILE
      sed -i "${addone}i</userdata>" $USER_DATA_FILE
      lastline=$(cat ${USER_DATA_FILE} | grep -n "</userdata>" | cut -d':' -f1)
   fi
   servtag="<server address=\"localhost\">"

   #set up executable strings
   if [ ! $(cat ${USER_DATA_FILE} | grep "executables" | tail -1) ]; then
      i=1
      for cmd in echo cp grep ls mkdir cat python perl; do
         tmp="var$i"
         export var$i=$(which $cmd)
         #if the which command fails, use default value
         if [ $(echo $?) != 0 ]; then
           export var$i="/bin/$cmd"
         fi
         export $cmd="<executable name=\"${cmd}\" version=\"*\" location=\"${!tmp}\" />"
         let i+=1
      done

      #write everything to correct place
      echo "writing executable values to ${USER_DATA_FILE}"
      sed -i "${lastline}i \ \ \ \ \ <executables>" ${USER_DATA_FILE}
      let lastline+=1
      sed -i "${lastline}i \\\t\ \ ${servtag}" ${USER_DATA_FILE}
      let lastline+=1
      for x in echo cp grep ls mkdir cat python perl; do
         sed -i "${lastline}i \\\t\t ${!x}" ${USER_DATA_FILE}
         let lastline+=1
      done
      sed -i "${lastline}i \\\t\ \ </server>" ${USER_DATA_FILE}
      let lastline+=1
      sed -i "${lastline}i \ \ \ \ \ </executables>" ${USER_DATA_FILE}
   fi
fi
###########CREATING jaas config FILE
echo "=======| Creating jaas config file"
echo "/** Login Configuration for the Pipeline **/

PipelineLogin {
   $LOGIN_MODULE_CLASS required debug=false;
};
">> $PL_LOCATION/dist/pipeline_jaas.config
chown $PL_USER $PL_LOCATION/dist/pipeline_jaas.config


###########CREATING LAUNCH SERVER SCRIPT
echo "=======| Creating launch Server Script"
echo '#!/bin/bash

SGE_ROOT="'$SGE_ROOT'"
BASEPATH="'$PL_LOCATION'"
PREFS_PATH=$BASEPATH

if [ -f ${BASEPATH}/running.pid ]
then
   echo "Server already running."
   exit 1;
fi


WHOAMI=`whoami`
if [ $WHOAMI = "root" ]
then
    echo "You are root. Please run this script only with user:'$PL_USER'"
    exit 1;
fi

# if the SGE_CELL environmental variable is not set, use the default value
if [ -z $SGE_CELL ]; then
   export SGE_CELL=default
fi

cd ${BASEPATH}

#set core dump file limit to unlimited
ulimit -c unlimited

# so the cache directories that get created by the server have 'rwx' for all
umask 000

if [ ! -z "`grep GridPluginJARFiles $PREFS_PATH/preferences.xml`" ]
then 
    if [ -d $SGE_ROOT ] && [ ! -z "$SGE_ROOT" ]
    then
        # setting LD_LIBRARY_PATH in case somebody runs the script using 'su' or 'sudo'
        arch=`${SGE_ROOT}/util/arch`
        export LD_LIBRARY_PATH=$SGE_ROOT/lib/${arch}:$BASEPATH/dist/:$LD_LIBRARY_PATH

        # reading in the SGE libraries
        source $SGE_ROOT/$SGE_CELL/common/settings.sh
    else
        echo "SGE_ROOT is not defined or set to an invalid directory."
        echo "Server not started."
        exit 2;
    fi
else
    export LD_LIBRARY_PATH=$BASEPATH/dist/:$LD_LIBRARY_PATH
fi

'$JAVA_HOME'/bin/java '$d64' -server '${memory}' -Djava.awt.headless=true -Djava.security.auth.login.config=$BASEPATH/dist/pipeline_jaas.config -cp .:$SGE_ROOT/lib/drmaa.jar'$LOGIN_MODULE':$BASEPATH/dist/Pipeline.jar server.Main -preferences $PREFS_PATH/preferences.xml > $BASEPATH/outputStream.log 2> $BASEPATH/errorStream.log&

if [ $? -eq 0 ]
then
	echo $! > ${BASEPATH}/running.pid
	echo "Server successfully started."
        exit 0
else
	echo "Failed to start server"
	exit 1
fi'>>$PL_LOCATION/launchServer.sh

chmod +x $PL_LOCATION/launchServer.sh
chown $PL_USER $PL_LOCATION/launchServer.sh

echo "=======| Creating kill Server Script"

###########CREATING KILL SERVER SCRIPT
echo '#! /bin/bash
BASEPATH="'$PL_LOCATION'"

PID_FILE=$BASEPATH/running.pid

if [ -f ${PID_FILE} ]
then

echo -n "Killing server..."

PID=`cat ${PID_FILE}`

kill -9 $PID 2>/dev/null

while ps -p $PID > /dev/null; do sleep 1; done

if [ $? -eq 0 ]
then
    rm ${BASEPATH}/running.pid
    echo "DONE"
    exit 0
else
    echo "FAILED"
    exit 1
fi
else
echo "No pipeline server running."
fi '>>$PL_LOCATION/killServer.sh

chmod +x $PL_LOCATION/killServer.sh
chown $PL_USER $PL_LOCATION/killServer.sh



if [ "$INSTALL_SUPERUSER" = "true" ]
then

    ##### PIPELINE CONFIG
    if [ -z "`grep PIPELINE_CONFIG /etc/sudoers`" ]
    then
        echo "=======| Modifying Sudoers file"
        requiretty=`grep -n requiretty /etc/sudoers | grep Defaults | cut -d':' -f1`
        if [ ! -z $requiretty ]; then
           sed -i ${requiretty}c"# Defaults requiretty" /etc/sudoers
        fi 
		echo "" >> /etc/sudoers
		echo -e "########## PIPELINE_CONFIG\nDefaults env_keep += \"PERL5OPT PERL5LIB PERLLIB JAVA_HOME SGE_ROOT SGE_CELL SGE_PORT SGE_CLUSTER_NAME\"
Defaults always_set_home" >> /etc/sudoers
        securelines=`grep -n Defaults[[:space:]]*secure_path /etc/sudoers | cut -d':' -f1`
        for secureline in $securelines
        do
           sed -i ${secureline}c"Defaults \!secure_path" /etc/sudoers
        done
        chmod 440 /etc/sudoers
    fi


    echo "=======| Making Pipeline power user"
    ##### LONIPipeline_${PL_USER}

    if [ ! -z "`grep LONIPipeline_ /etc/sudoers`" ]
    then
        LN=`grep -n "LONIPipeline_" /etc/sudoers | cut -f1 -d:`
        echo LN=$LN
        END_LN=$(( $LN + 6 ))
        echo END_LN=$END_LN
        sed -i ${LN},${END_LN}d /etc/sudoers
        
    fi


    SUPERUSERS="!root"

    if [ ! -z "$SUPERUSER_LIST" ]
    then
        SUPERUSERS+=",!$SUPERUSER_LIST"
    fi

    echo "Modifying sudoers list for $PL_USER"
    echo -e "########## LONIPipeline_${PL_USER}\nHost_Alias CLUSTER = ${PL_HOSTNAME}\nUser_Alias PIPELINE1 = ${PL_USER}\nUser_Alias PIPELINE2 = ALL,$SUPERUSERS\nRunas_Alias PIPELINE2_RUN = ALL,$SUPERUSERS\nPIPELINE1 CLUSTER = (PIPELINE2_RUN) NOPASSWD: ALL\n" >> /etc/sudoers

fi

if [ $PL_START_ON_STARTUP = "true" ]
then
    echo "=======| Creating Pipeline startup script"
    if [ -f /etc/init.d/pipeline ]
    then
        rm -f /etc/init.d/pipeline
    fi
    ###########CREATING START SERVER ON STARTUP SCRIPT
    echo '#!/bin/bash
### BEGIN INIT INFO
# Provides:          pipeline
# Required-Start:    $network
# Required-Stop:
# Should-Start: 
# Default-Start:     3 5
# Default-Stop:      0 1 2 6
# Short-Description: controls Pipeline server on boot+shutdown
# Description: Brings up the Pipeline server on boot,
#       takes it offline on shutdown/restart.
### END INIT INFO

    sge_source=/etc/profile.d/sge.sh
    if [ -a $sge_source ]; then
       source $sge_source
    fi

    SERVER_PATH="'$PL_LOCATION'"
    DB_PATH="'$PLDB_LOCATION'"
    PIPELINE_USER="'$PL_USER'"

    case "$1" in
      start)
        WHOAMI=`whoami`
        if [ $WHOAMI = "root" ]
        then
          if [ -f ${SERVER_PATH}/running.pid ]; then
             echo "Deleting defunct ${SERVER_PATH}/running.pid file..."
             /bin/mv ${SERVER_PATH}/running.pid /tmp
          fi
          sudo -u ${PIPELINE_USER} ${DB_PATH}/startDB.sh
          sudo -u ${PIPELINE_USER} ${SERVER_PATH}/launchServer.sh

          exit $?
        else
          echo "Insufficient privileges."
          exit 1
        fi
        ;;
      stop)
        WHOAMI=`whoami`
        if [ $WHOAMI = "root" ]
        then
          if [ -f ${SERVER_PATH}/running.pid ]; then
            echo "Stopping the Pipeline Database..."
            sudo -u ${PIPELINE_USER} ${DB_PATH}/stopDB.sh
            echo "Stopping the Pipeline Server..."
            sudo -u ${PIPELINE_USER} ${SERVER_PATH}/killServer.sh
          else
            echo "Pipeline Server not running."
            exit 1
          fi
        else
          echo "Insufficient privileges."
          exit 1
        fi
        ## If somehow these file were not deleted, delete them
        if [ -f ${SERVER_PATH}/running.pid ]; then
          echo "Deleting ${SERVER_PATH}/running.pid..."
          rm -f ${SERVER_PATH}/running.pid
        fi
        exit 0
        ;;
      *)
        ## If no parameters are given, print which are avaiable.
        echo "Usage: $0 {start|stop}"
        exit 1
        ;;
    esac
    '>>/etc/init.d/pipeline

    chmod +x /etc/init.d/pipeline

    if [ ! -f /sbin/chkconfig ]; then
        if [ "$debian" = "true" ]; then
            update-rc.d pipeline start 50 3 5 . stop 50 0 1 2 6 .
        fi
    else 
	    /sbin/chkconfig --add pipeline
		/sbin/chkconfig pipeline on
	fi

fi

##################################################
#           INSTALLING TEST IMAGE FILE
##################################################
echo "=======| Downloading test image files"
echo "pl-->Downloading: "
echo "pl->Test input files"

# download test input files
wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/test_inputs.tar.gz

echo "pl++";

# decompress test input files
tar xmvzf test_inputs.tar.gz
echo "pl++";
##################################################
#           INSTALLING UTILITIES
##################################################
echo "=======| Installing utilities"
echo "pl-->Installing: "
echo "pl->Utilities"

# download smartline archive
wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/smartline.tar.gz
wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/idaget.tar.gz
wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/xnat.tar.gz
wget -c --progress=dot http://pipeline.loni.usc.edu/files/dps/thumbgen.tar.gz

echo "pl++";

mkdir -p $TOOLS_PATH/PipelineUtilities
chown $PL_USER $TOOLS_PATH/PipelineUtilities

cp smartline.tar.gz idaget.tar.gz xnat.tar.gz thumbgen.tar.gz $TOOLS_PATH/PipelineUtilities
cd $TOOLS_PATH/PipelineUtilities

# decompress utility archives
tar xmvzf smartline.tar.gz
tar xmvzf idaget.tar.gz
tar xmvzf xnat.tar.gz
tar xmvzf thumbgen.tar.gz

# clean archives
rm smartline.tar.gz idaget.tar.gz xnat.tar.gz thumbgen.tar.gz

echo "pl++";

##################################################
#           CONFIGURE SGE
##################################################

if [ -z "`grep SGE_CONFIG /etc/profile`" ]
then
    echo -e "########## SGE_CONFIG\nexport SGE_ROOT=$SGE_ROOT\n">> /etc/profile
elif [ -z "`grep SGE_ROOT /etc/profile | cut -d'=' -f2`" ]
then
   sed -i "s|SGE_ROOT=|SGE_ROOT=$SGE_ROOT|g" /etc/profile 
fi

if [ -f /etc/csh.login ]; then
   if [ -z "`grep SGE_CONFIG /etc/csh.login`" ]
   then
      echo -e "########## SGE_CONFIG\nsetenv SGE_ROOT $SGE_ROOT\n">> /etc/csh.login
   elif [ -z "`grep SGE_ROOT /etc/profile | cut -d'=' -f2`" ]
   then
      sed -i "s|SGE_ROOT=|SGE_ROOT=$SGE_ROOT|g" /etc/csh.login
   fi
fi

#################################################
#     Check KexAlgorithms for SSH on Debian
#################################################
if [ "$debian" == "true" ]; then
    if [ "$(grep Kex /etc/ssh/sshd_config | grep diffie)" ]; then
       echo "Appropriate alogorithms found"
    else 
       echo "KexAlgorithms diffie-hellman-group1-sha1,diffie-hellman-group-exchange-sha1" >> /etc/ssh/sshd_config 
       echo -n "Restarting ssh service..."
       service $ssh restart
       echo "done."
    fi 
fi


echo "=======| PIPELINE INSTALLATION COMPLETE |======="
echo "pl++"
echo "pl-->Complete"
echo "pl->"
