#!/bin/bash

if [ -z "$INSTALL_SGE" ]
then
    echo "Please do not run this script manually. Run launchInstaller.sh instead."
    exit 1
fi

#check redhat etc
if [ -a /etc/redhat-release ]; then
    redhat=true
    install='yum -y install'
	motif='openmotif'
    if [ "`grep -i fedora /etc/redhat-release`" ]; then 
		install='dnf -y install'
        motif='openmotif'
	fi
elif [ "$(grep -is debian /etc/*-release)" ]; then
    debian=true
	install='apt-get -y install'
    motif='libmotif-common'
fi

arch=`arch`
if [ "$arch" == "x86_64" ] 
then
    sge_filename="sge-8.1.8-lx-amd64"
    hwloc_filename="libhwloc.so.4.tar.gz"
    if [ "$redhat" == "true" ]; then
        sge_filename_os="sge-8.1.8-lx-amd64_cent"
        #sge_filename="sge-8.1.7a-lx-amd64"
        lib="lib64"
		$install epel-release
	    $install jemalloc
    elif [ "$debian" == "true" ]; then
        sge_filename_os="${sge_filename}_deb"
        lib="lib/x86_64-linux-gnu"
        if [ ! "$(ldconfig -p | grep libjemalloc)" ]; then
	        $install libjemalloc-dev
        fi
    fi
else
    echo "sge!Can't install on this OS as 64bit system is required."
    echo "Exiting the installer as 64bit system requirement is not met."
    exit 1
fi 

#echo "sge=>20"

# add /usr/sbin and /sbin to PATH
export PATH=${PATH}:/usr/sbin:/sbin

umask 0002

DL_DIR=$(cd `dirname $0` && pwd)

sge_conf_file=${SGE_ROOT}/util/install_modules/dps_configuration.conf

echo "sge->";
echo "sge-->Checking for previous installations...";
sgedirs=$(${DL_DIR}/checkSGE.sh | grep SGE | grep -v " SGE")
numsgedirs=$(echo $sgedirs | wc -l)
if [ $numsgedirs -eq 1 ]
then
	sgedir=$(echo $sgedirs | cut -d' ' -f2)
fi
echo "sge++"

if [ ! -z "${sgedir}" ] && [ -d ${sgedir} ]
then
    echo "sge-->Uninstalling previous installation..."
    cdir=$(pwd)
    export SGE_ROOT=$sgedir
    cd $SGE_ROOT
    ${sgedir}/inst_sge -ux -auto ${sge_conf_file}
    ${sgedir}/inst_sge -um -auto ${sge_conf_file}
    rm -Rf /etc/init.d/sge*
    rm -Rf ${sgedir}/spool
    rm -Rf ${sgedir}
    cd $cdir
fi

echo "sge++"

echo "sge-->"


echo "sge-->Installing Prerequisites: "
echo "sge->Openmotif"

eval $install $motif
echo "sge+=3"

# restore SGE_ROOT for new installation
mkdir -p ${SGE_ROOT}


echo "sge-->Configuring Installation files: "
echo "sge->/etc/profile.d/sge.sh"
##### SGE CONFIG

if [ ! -a /etc/profile.d/sge.sh ]; then 
	touch /etc/profile.d/sge.sh
fi

for file in /etc/profile /etc/profile.d/sge.sh /etc/bash.bashrc; do
   if [ -a ${file} ]; then
      if [ -z "`grep SGE_CONFIG ${file}`" ]
         then
         echo -e "########## SGE_CONFIG\nexport SGE_ROOT=$SGE_ROOT\n. ${SGE_ROOT}/${SGE_CELL_NAME}/common/settings.sh">> $file
      else
         sgeroot_lineNumber=$(grep -n "export SGE_ROOT" $file | cut -d':' -f1)
         sed -i ${sgeroot_lineNumber}c"export SGE_ROOT=$SGE_ROOT" $file
         sourcesettings_lineNumber=$[sgeroot_lineNumber+1]
         sed -i ${sourcesettings_lineNumber}c". ${SGE_ROOT}/${SGE_CELL_NAME}/common/settings.sh" $file
      fi
   fi
done

if [ -f /etc/csh.login ]; then
	echo "sge->/etc/csh.login"
	if [ -z "`grep SGE_CONFIG /etc/csh.login`" ]
	then
		echo -e "########## SGE_CONFIG\nsetenv SGE_ROOT $SGE_ROOT\n. ${SGE_ROOT}/${SGE_CELL_NAME}/common/settings.csh">> /etc/csh.login
	else
		sgeroot_lineNumber=$(grep -n "setenv SGE_ROOT" /etc/csh.login | cut -d':' -f1)
		sed -i ${sgeroot_lineNumber}c"setenv SGE_ROOT $SGE_ROOT" /etc/csh.login
		sourcesettings_lineNumber=$[sgeroot_lineNumber+1]
		sed -i ${sourcesettings_lineNumber}c". ${SGE_ROOT}/${SGE_CELL_NAME}/common/settings.csh" /etc/csh.login
	fi
fi

echo "sge++"

echo "sge-->Installing: "
echo "sge->hwloc library"

wget -c --progress=dot  ${server}/${hwloc_filename}
echo "sge+=2"

mv ${hwloc_filename} /usr/${lib}
cd /usr/${lib}
tar xmvzf ${hwloc_filename}


if [ "$arch" == "x86_64" ]
then
    if [ -f libhwloc.so.5 ]; then 
        echo "libhwloc.so.5 exists. Not creating link to previous version"
    else
        ln -s libhwloc.so.4 libhwloc.so.5
    fi
fi

rm ${hwloc_filename}
echo "sge++"

#################################################
#           KILL RUNNING SGE PROCESSES
#################################################
echo "Checking for running sge processes..."
IFS='
'
for sge_component in sge_qmaster sge_execd; do
   echo "checking $sge_component"
   sub_process=""
   process_num=""
   for process in $(ps -ef | grep $sge_component); do
      if [ -z "$(echo $process | grep "grep")" ]; then
         sub_process=$process
         echo "process: $sub_process"
         process_num=$(echo $sub_process | tr -s ' ' | cut -d' ' -f2)
         echo "process number: $process_num"
      fi
   done
   if [ ! -z "$process_num" ]; then
      echo "Killing process $process_num"
      kill -9 $process_num
   fi
done
IFS=' ' 


##################################################
#           DOWNLOAD AND CONFIGURE
##################################################
echo "sge-->Downloading: "
echo "sge->SGE Core files"
cd $SGE_ROOT
wget -c --progress=dot ${server}/${sge_filename_os}.tar.gz
echo "sge-->Extracting: "
echo "sge+=3"
tar -xzvf ${sge_filename_os}.tar.gz
echo "sge+=2"
cp -R ${sge_filename}/* ./
rm -rf ${sge_filename} ${sge_filename_os}.tar.gz
echo "sge++"

echo "sge-->Configuring: "
echo "sge->SGE Installation files"


cp ${SGE_ROOT}/util/install_modules/inst_template.conf ${sge_conf_file}
echo "sge++"
# remove unnecessary lines from config file
begin=`grep -n JMX ${sge_conf_file} | head -n 1 | cut -d':' -f1`
end=`grep -n JVM ${sge_conf_file} | tail -n 1 | cut -d':' -f1`
first=$(expr $begin - 1 )
lines=`wc ${sge_conf_file} | cut -d' ' -f3`
last=$(expr $lines - $end )
head -n $first ${sge_conf_file} > ${SGE_ROOT}/util/install_modules/tmp.conf
tail -n $last ${sge_conf_file} >> ${SGE_ROOT}/util/install_modules/tmp.conf
mv -f ${SGE_ROOT}/util/install_modules/tmp.conf ${sge_conf_file}


# update auto configuration file
adminuser_line=`grep -n ADMIN_USER= ${sge_conf_file} | cut -d':' -f1`
sed -i ${adminuser_line}c"ADMIN_USER=\"${SGE_ADMIN_USER}\"" ${sge_conf_file}

cellname_line=`grep -n CELL_NAME= ${sge_conf_file} | cut -d':' -f1`
sed -i ${cellname_line}c"CELL_NAME=\"${SGE_CELL_NAME}\"" ${sge_conf_file}

sgeroot_line=`grep -n SGE_ROOT= ${sge_conf_file} | cut -d':' -f1`
sed -i ${sgeroot_line}c"SGE_ROOT=\"${SGE_ROOT}\"" ${sge_conf_file}

qmasterspool_line=`grep -n QMASTER_SPOOL_DIR= ${sge_conf_file} | cut -d':' -f1`
sed -i ${qmasterspool_line}c"QMASTER_SPOOL_DIR=\"${SGE_ROOT}/${SGE_CELL_NAME}/spool/qmaster\"" ${sge_conf_file}

execdspool_line=`grep -n EXECD_SPOOL_DIR= ${sge_conf_file} | cut -d':' -f1`
sed -i ${execdspool_line}c"EXECD_SPOOL_DIR=\"${SGE_ROOT}/${SGE_CELL_NAME}/spool\"" ${sge_conf_file}

dbspool_line=`grep -n DB_SPOOLING_DIR= ${sge_conf_file} | cut -d':' -f1`
sed -i ${dbspool_line}c"DB_SPOOLING_DIR=\"${SGE_ROOT}/${SGE_CELL_NAME}/db\"" ${sge_conf_file}

sgecluster_line=`grep -n SGE_CLUSTER_NAME= ${sge_conf_file} | cut -d':' -f1`
sed -i ${sgecluster_line}c"SGE_CLUSTER_NAME=\"${SGE_CLUSTER_NAME}\"" ${sge_conf_file}

submithost_line=`grep -n SUBMIT_HOST_LIST= ${sge_conf_file} | cut -d':' -f1`
sed -i ${submithost_line}c"SUBMIT_HOST_LIST=\"${SGE_SUBMIT_HOSTS}\"" ${sge_conf_file}

EXECUTIONHOSTS=$(echo "$SGE_EXEC_HOSTS" | tr ',' ' ')

exechost_line=`grep -n EXEC_HOST_LIST= ${sge_conf_file} | cut -d':' -f1`
exechostremove_line=`grep -n EXEC_HOST_LIST_RM= ${sge_conf_file} | cut -d':' -f1`
sed -i ${exechost_line}c"EXEC_HOST_LIST=\"${EXECUTIONHOSTS}\"" ${sge_conf_file}
sed -i ${exechostremove_line}c"EXEC_HOST_LIST_RM=\"${EXECUTIONHOSTS}\"" ${sge_conf_file}

adminhost_line=`grep -n ADMIN_HOST_LIST= ${sge_conf_file} | cut -d':' -f1`
sed -i ${adminhost_line}c"ADMIN_HOST_LIST=\"${SGE_ADMIN_HOSTS}\"" ${sge_conf_file}

localspool_line=`grep -n EXECD_SPOOL_DIR_LOCAL= ${sge_conf_file} | cut -d':' -f1`
sed -i ${localspool_line}c"EXECD_SPOOL_DIR_LOCAL=\"${SGE_SPOOL_DIR}\"" ${sge_conf_file}

if [ -z "`grep sge_qmaster /etc/services`" ]
then
    echo "
    sge_qmaster     6444/tcp                        
    sge_execd       6445/tcp                        " >> /etc/services
fi

echo "sge-->Installing: "
echo "sge->SGE"
echo "sge++"

# Start the installation
echo "Installing SGE..."
./inst_sge -m -x -auto ${sge_conf_file}
#update-rc.d sgemaster.cluster start 50 3 5 . stop 50 0 1 2 6 .
if [ $? -ne 0 ]; then
    echo "sge!Failed to install SGE"
    exit 1
fi

echo "sge-->Configuring: "
echo "sge->SGE Scheduler"

source $SGE_ROOT/$SGE_CELL_NAME/common/settings.sh

#Configure the scheduler so it prints the job info.
qconf -ssconf > /tmp/sconf.txt
scheduler_line=`grep -n schedd_job_info /tmp/sconf.txt | cut -d':' -f1`
sed -i ${scheduler_line}c"schedd_job_info true" /tmp/sconf.txt
qconf -Msconf /tmp/sconf.txt
if [ $? -ne 0 ]; then
	echo "sge!Failed to configure SGE scheduler"
	exit 1
fi
rm /tmp/sconf.txt
echo "sge++"
if [ $? -eq 0 ]
then
    echo "sge->Success"
else
    echo "sge->Failed"
fi

if [ $SGE_CONFIG_QUEUE = "true" ]
then
        echo "sge-->Creating queue: "
        echo "sge->${SGE_QUEUE_NAME}"
	
        QUEUE_SLOTS=${SGE_QUEUE_SLOTS//","/" "}

	# create a new host list file with specified slots
	qconf -shgrp @allhosts > ${SGE_ROOT}/${SGE_QUEUE_HOSTLIST}
	sed -i 1c"group_name ${SGE_QUEUE_HOSTLIST}" ${SGE_ROOT}/${SGE_QUEUE_HOSTLIST}
	sed -i 2c"hostlist ${QUEUE_SLOTS}" ${SGE_ROOT}/${SGE_QUEUE_HOSTLIST}

	# modify slots string for auto queue config file
	tmp=`echo [${QUEUE_SLOTS}=1] | tr -s ' '`
	queue_slots=${tmp//[[:space:]]/'=1],['}

	# create a new queue configuration file with specified host group, slots
	qconf -sq all.q > ${SGE_ROOT}/${SGE_QUEUE_NAME}
	sed -i "s/all.q/${SGE_QUEUE_NAME}/g" ${SGE_ROOT}/${SGE_QUEUE_NAME}
	sed -i "s/@allhosts/${SGE_QUEUE_HOSTLIST}/g" ${SGE_ROOT}/${SGE_QUEUE_NAME}
	slotsline=`grep -n slots ${SGE_ROOT}/${SGE_QUEUE_NAME} | cut -d':' -f1`
	sed -i ${slotsline}c"slots                 1,${queue_slots}" ${SGE_ROOT}/${SGE_QUEUE_NAME}

	# add host group
	qconf -Ahgrp ${SGE_QUEUE_HOSTLIST}
	# add queue
	qconf -Aq ${SGE_QUEUE_NAME}
fi

echo "sge++"


echo "sge-->Finalizing installation..."
echo "sge->"
	
# show new host group list and queue list
echo "New host group list"
qconf -shgrpl
echo "New queue list"
qconf -sql

source ${SGE_ROOT}/${SGE_CELL_NAME}/common/settings.sh

#++++++++++++++ INCREASE PROGRESS
echo "sge++"
echo "sge-->Complete"
echo "sge->"
echo "SGE Installation Complete"
