#!/bin/bash

args=$@

if [ "`whoami`" = "root" ]
then

    #check redhat (includes centos)
	dist=$(grep -is "PRETTY" /etc/*release | cut -d'=' -f2)
	if [ -z "$dist" ]; then
		dist=$(cat /etc/*-release 2>/dev/null | head -1 | cut -d'=' -f2)
	fi
	echo "Installing on ${dist}..."
    if [ -a /etc/redhat-release ]; then
        get="yum"
        redhat=true
		if [ "`grep -is fedora /etc/redhat-release`" ]; then
			get="dnf"
		fi
    elif [ "$(grep -is debian /etc/*-release)" ]; then
        get="apt-get"
	    debian=true
    fi

	#if [  ! -f /etc/redhat-release ] || [ -z "`cat /etc/redhat-release | grep CentOS`" ]
    if [ "$debian" != true ] && [ "$redhat" != true ] && [ "$opensuse" != true ]; then
		echo ""
		echo "     WARNING: Pipeline Server Installer is designed to run on Linux RedHat derivatives and Debian"
		echo ""
		echo "     Using this installer on this OS may DAMAGE YOUR SYSTEM."
		echo ""
		echo -n "     Are you sure you want to start the installer anyway [y/n] ?"
	        read yesno
		
		if [ "$yesno" = "y" ] || [ "$yesno" = "yes" ]
		then
			echo "Launching the installer..."
		else
			exit 1
		fi
	fi

        head="-Djava.awt.headless=false" 
        if [ $# -ne 0 ]
        then
            printusage=false
            if [ $1 != "-auto" ] && [ $1 != "-interactive" ]
            then
                printusage=true
            fi

	    if [ $1 = "-auto" ] && [ $# -ne 2 ]
            then 
               printusage=true
            fi

            if [ "$printusage" == "true" ]
            then
                echo "Usage: ./launchInstaller.sh [-auto installPreferences.xml]"
                echo "Running the script without any arguments will launch the GUI."
                echo "The optional -auto flag, followed by a valid installation preferences"
                echo "file will start the installation in automatic, headless mode."
                exit 1
            fi

            if [ ! -f $2 ]
            then
                echo "The installation preferences file does not exist."
                echo "Please launch the installer again with a valid file."
                exit 1
            else
                # check if the user has specified a relative path for the config file
                firstchar=${2:0:1}
                if [ "$firstchar" != "/" ]
                then
                    args="$1 `pwd`/$2"
                fi
                head="-Djava.awt.headless=true"
            fi
        fi


	which java > /dev/null 2> /dev/null
	if [ $? != 0 ] 
	then
		echo "Missing JAVA"
		echo "The installer requires Oracle JAVA. Please follow these instructions to install Oracle JAVA, then launch the installer again."
		echo "1. Visit the following URL: http://www.oracle.com/technetwork/java/javase/downloads/index.html"
                echo "2. Click on Download JDK button."
                if [ $redhat = true ]; then
                   echo "3. Click on jdk-[ver]-linux-[platform].rpm link to begin download."
                   echo "4. Run rpm --prefix=INSTALL_DIR -ivh RPM_FILE_PATH"
                elif [ $debian = true ]; then
                   echo "3. Click on jdk-[ver]-linux-[platform].tar.gz link to begin download."
                   echo "4. Run tar --directory=INSTALL_DIR -zxvf TAR_FILE_PATH"
                fi
                echo "5. Run /usr/sbin/update-alternatives --install /usr/bin/java java INSTALL_DIR/JAVA_DIR/bin/java 2"
                echo "6. Run /usr/sbin/update-alternatives --set java INSTALL_DIR/JAVA_DIR/bin/java"
                exit 3
	fi

        which make > /dev/null 2> /dev/null
        if [ $? != 0 ]
        then
                echo "Missing make"
                echo "The installer requires GNU make. It will install make before proceeding with the rest of the installation."
                $get -y install make
        fi

        # determine firewall file, which varies between CentOS versions 5 and 6
        firewall_file=system-config-securitylevel
        if [ ! -f /etc/sysconfig/${firewall_file} ]
        then
            firewall_file=system-config-firewall
        fi

        # create backup files
        cp /etc/sudoers /etc/profile /etc/csh.login /etc/services /etc/sysconfig/${firewall_file} /tmp 2>/dev/null
        cp --backup=numbered /tmp/sudoers /etc 2>/dev/null
        cp --backup=numbered /tmp/profile /etc 2>/dev/null
        cp --backup=numbered /tmp/csh.login /etc 2>/dev/null
        cp --backup=numbered /tmp/services /etc 2>/dev/null
        cp --backup=numbered /tmp/${firewall_file} /etc/sysconfig 2>/dev/null

	if [ -d ./dist ]; then cd dist; fi
	
	if [ "$1" = "-interactive" ]; then
	   ./create_installPref.sh
	else
	   java -Xmx512m -jar $head PipelineServerInstaller.jar $args
	fi
else
	echo "Permission denied: You have to be root to launch the Pipeline Installer"
	exit 1
fi
